import { copyToClipboard } from './helpers';

// Mock the necessary DOM methods and objects
global.navigator.clipboard = {
  writeText: jest.fn().mockResolvedValue(undefined)
};

const mockWriteText = jest.spyOn(navigator.clipboard, 'writeText');
const mockConsoleError = jest.spyOn(console, 'error').mockImplementation(() => {});

document.getElementById = jest.fn();

describe('copyToClipboard', () => {
  beforeEach(() => {
    // Clear all mocks before each test
    jest.clearAllMocks();
  });

  test('should copy input value to clipboard', () => {
    const mockInput = { value: 'Test input value', tagName: 'INPUT' };
    document.getElementById.mockReturnValue(mockInput);

    copyToClipboard('testInputId');

    expect(mockWriteText).toHaveBeenCalledWith('Test input value');
    expect(console.error).not.toHaveBeenCalled();
  });

  test('should copy div innerText to clipboard', () => {
    const mockDiv = { innerText: 'Test div text', tagName: 'DIV' };
    document.getElementById.mockReturnValue(mockDiv);

    copyToClipboard('testDivId');

    expect(mockWriteText).toHaveBeenCalledWith('Test div text');
    expect(console.error).not.toHaveBeenCalled();
  });

  test('should handle unsupported element types', () => {
    const mockElement = { tagName: 'SPAN' };
    document.getElementById.mockReturnValue(mockElement);

    copyToClipboard('testSpanId');

    expect(mockWriteText).toHaveBeenCalledWith(undefined);
    expect(console.error).not.toHaveBeenCalled();
  });

  test('should handle clipboard write errors', async () => {
    const mockInput = { value: 'Test input value', tagName: 'INPUT' };
    document.getElementById.mockReturnValue(mockInput);
    mockWriteText.mockRejectedValue(new Error('Clipboard write failed'));

    await copyToClipboard('testInputId');

    expect(mockWriteText).toHaveBeenCalledWith('Test input value');
    expect(console.error).toHaveBeenCalledWith('Failed to copy text: ', expect.any(Error));
  });

  test('should handle null element', () => {
    document.getElementById.mockReturnValue(null);

    expect(() => {
      copyToClipboard('nonExistentId');
    }).toThrow(TypeError);

    expect(mockWriteText).not.toHaveBeenCalled();
    expect(console.error).not.toHaveBeenCalled();
  });
});