import { createItem } from './helpers';

// Mock DOM elements and functions
document.querySelector = jest.fn();
document.createRange = jest.fn(() => ({
  createContextualFragment: jest.fn((html) => {
    const div = document.createElement('div');
    div.innerHTML = html;
    return div.firstChild;
  })
}));

describe('createItem', () => {
  let mockWrapper, mockInput, mockSelect, mockCreateContextualFragment;

  beforeEach(() => {
    jest.clearAllMocks();

    mockWrapper = {
      closest: jest.fn().mockReturnThis(),
      querySelector: jest.fn(),
      parentElement: {
        querySelector: jest.fn()
      },
      insertBefore: jest.fn()
    };
    mockInput = { value: '' };
    mockSelect = document.createElement('div');
    mockCreateContextualFragment = jest.fn((html) => {
      const div = document.createElement('div');
      div.innerHTML = html;
      return div.firstChild;
    });

    mockWrapper.parentElement.querySelector.mockReturnValue(mockInput);
    mockWrapper.querySelector.mockReturnValue(mockSelect);
    document.createRange().createContextualFragment = mockCreateContextualFragment;
  });

  test('should create a new item when value does not exist', () => {
    const value = 'newItem';
    const options = {
      selector: '.selector',
      selectorWrapper: '.wrapper',
      valueSelector: '.value-selector'
    };
    const inputElm = { closest: jest.fn().mockReturnValue(mockWrapper) };
  
    createItem(value, options, inputElm);
  
    expect(inputElm.closest).toHaveBeenCalledWith('.wrapper');
    expect(mockWrapper.parentElement.querySelector).toHaveBeenCalledWith('.value-selector');
    expect(mockWrapper.querySelector).toHaveBeenCalledWith('.selector');
    expect(document.createRange).toHaveBeenCalled();
    expect(mockWrapper.insertBefore).toHaveBeenCalled();
    expect(mockInput.value).toBe(value);
  });

  test('should append value when input already has a value', () => {
    const existingValue = 'existingItem';
    const newValue = 'newItem';
    mockInput.value = existingValue;

    const options = {
      selector: '.selector',
      selectorWrapper: '.wrapper',
      valueSelector: '.value-selector'
    };
    const inputElm = { closest: jest.fn().mockReturnValue(mockWrapper) };

    createItem(newValue, options, inputElm);

    expect(mockInput.value).toBe(`${existingValue},${newValue}`);
  });

  test('should not create a new item when value already exists', () => {
    const existingValue = 'existingItem';
    mockInput.value = existingValue;

    const options = {
      selector: '.selector',
      selectorWrapper: '.wrapper',
      valueSelector: '.value-selector'
    };
    const inputElm = { closest: jest.fn().mockReturnValue(mockWrapper) };

    createItem(existingValue, options, inputElm);

    expect(mockCreateContextualFragment).not.toHaveBeenCalled();
    expect(mockWrapper.insertBefore).not.toHaveBeenCalled();
    expect(mockInput.value).toBe(existingValue);
  });

  test('should log error when wrapper is not found', () => {
    const value = 'newItem';
    const options = {
      selector: '.selector',
      selectorWrapper: '.wrapper',
      valueSelector: '.value-selector'
    };
    const inputElm = { closest: jest.fn().mockReturnValue(null) };
    console.error = jest.fn();

    createItem(value, options, inputElm);

    expect(console.error).toHaveBeenCalledWith('Wrapper not found:', '.wrapper');
  });

  test('should log error when hidden input is not found', () => {
    const value = 'newItem';
    const options = {
      selector: '.selector',
      selectorWrapper: '.wrapper',
      valueSelector: '.value-selector'
    };
    const inputElm = { closest: jest.fn().mockReturnValue(mockWrapper) };
    mockWrapper.parentElement.querySelector.mockReturnValue(null);
    console.error = jest.fn();

    createItem(value, options, inputElm);

    expect(console.error).toHaveBeenCalledWith('Hidden input not found:', '.value-selector');
  });

  test('should log error when select element is not found', () => {
    const value = 'newItem';
    const options = {
      selector: '.selector',
      selectorWrapper: '.wrapper',
      valueSelector: '.value-selector'
    };
    const inputElm = { closest: jest.fn().mockReturnValue(mockWrapper) };
    mockWrapper.querySelector.mockReturnValue(null);
    console.error = jest.fn();

    createItem(value, options, inputElm);

    expect(console.error).toHaveBeenCalledWith('Select element not found:', '.selector');
  });
});