import { handleRevealConditions } from './reveal';

// Mock DOM methods and properties
document.querySelector = jest.fn();
Element.prototype.querySelector = jest.fn();
Element.prototype.querySelectorAll = jest.fn();
Element.prototype.closest = jest.fn();

describe('handleRevealConditions', () => {
  let mockFieldset;

  beforeEach(() => {
    jest.clearAllMocks();
    mockFieldset = {
      querySelectorAll: jest.fn().mockReturnValue([])
    };
  });

  test('should hide non-matching elements and show matching element', () => {
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'target1', Value: 'value1' },
        { Target: 'target2', Value: 'value2' }
      ])),
      value: 'value1',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockTargetElement1 = { 
      style: { display: 'none' },
      setAttribute: jest.fn(), 
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([])
    };
    const mockTargetElement2 = { 
      style: { display: 'none' },
      setAttribute: jest.fn(), 
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([])
    };

    mockFieldset.querySelectorAll.mockImplementation((selector) => {
      if (selector === '.target1') return [mockTargetElement1];
      if (selector === '.target2') return [mockTargetElement2];
      return [];
    });

    handleRevealConditions(mockSelectElement);

    expect(mockTargetElement1.style.display).toBe('');
    expect(mockTargetElement1.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockTargetElement2.style.display).toBe('none');
    expect(mockTargetElement2.setAttribute).toHaveBeenCalledWith('data-reveal-target', '');
  });

  test('should handle nested select elements', () => {
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'target1', Value: 'value1' }
      ])),
      value: 'value1',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockNestedSelect = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'nestedTarget', Value: 'nestedValue' }
      ])),
      value: 'nestedValue',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockTargetElement = {
      style: { display: 'none' },
      setAttribute: jest.fn(),
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([mockNestedSelect]),
    };

    const mockNestedTargetElement = { 
      style: { display: 'none' },
      setAttribute: jest.fn(), 
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([])
    };

    mockFieldset.querySelectorAll.mockImplementation((selector) => {
      if (selector === '.target1') return [mockTargetElement];
      if (selector === '.nestedTarget') return [mockNestedTargetElement];
      return [];
    });

    handleRevealConditions(mockSelectElement);

    expect(mockTargetElement.style.display).toBe('');
    expect(mockTargetElement.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockNestedTargetElement.style.display).toBe('');
    expect(mockNestedTargetElement.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
  });

  test('should hide all elements when no matching condition is found', () => {
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'target1', Value: 'value1' },
        { Target: 'target2', Value: 'value2' }
      ])),
      value: 'value3',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockTargetElement1 = { 
      style: { display: 'none' },
      setAttribute: jest.fn(), 
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([])
    };
    const mockTargetElement2 = { 
      style: { display: 'none' },
      setAttribute: jest.fn(), 
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([])
    };

    mockFieldset.querySelectorAll.mockImplementation((selector) => {
      if (selector === '.target1') return [mockTargetElement1];
      if (selector === '.target2') return [mockTargetElement2];
      return [];
    });

    handleRevealConditions(mockSelectElement);

    expect(mockTargetElement1.style.display).toBe('none');
    expect(mockTargetElement1.setAttribute).toHaveBeenCalledWith('data-reveal-target', '');
    expect(mockTargetElement2.style.display).toBe('none');
    expect(mockTargetElement2.setAttribute).toHaveBeenCalledWith('data-reveal-target', '');
  });

  test('should handle missing target elements gracefully', () => {
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'target1', Value: 'value1' },
        { Target: 'target2', Value: 'value2' }
      ])),
      value: 'value1',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    mockFieldset.querySelectorAll.mockReturnValue([]);

    expect(() => handleRevealConditions(mockSelectElement)).not.toThrow();
  });

  test('should handle JSON parsing errors gracefully', () => {
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue('invalid JSON'),
      value: 'value1',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    console.error = jest.fn();

    expect(() => handleRevealConditions(mockSelectElement)).not.toThrow();
    expect(console.error).toHaveBeenCalledWith(expect.stringContaining('Error parsing JSON:'), expect.any(SyntaxError));
  });

  test('should handle multiple levels of nested select elements', () => {
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'target1', Value: 'value1' }
      ])),
      value: 'value1',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockNestedSelect1 = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'nestedTarget1', Value: 'nestedValue1' }
      ])),
      value: 'nestedValue1',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockNestedSelect2 = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify([
        { Target: 'nestedTarget2', Value: 'nestedValue2' }
      ])),
      value: 'nestedValue2',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockTargetElement = {
      style: { display: 'none' },
      setAttribute: jest.fn(),
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([mockNestedSelect1]),
    };

    const mockNestedTargetElement1 = {
      style: { display: 'none' },
      setAttribute: jest.fn(),
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([mockNestedSelect2]),
    };

    const mockNestedTargetElement2 = {
      style: { display: 'none' },
      setAttribute: jest.fn(),
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([]),
    };

    mockFieldset.querySelectorAll.mockImplementation((selector) => {
      if (selector === '.target1') return [mockTargetElement];
      if (selector === '.nestedTarget1') return [mockNestedTargetElement1];
      if (selector === '.nestedTarget2') return [mockNestedTargetElement2];
      return [];
    });

    handleRevealConditions(mockSelectElement);

    expect(mockTargetElement.style.display).toBe('');
    expect(mockTargetElement.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockNestedTargetElement1.style.display).toBe('');
    expect(mockNestedTargetElement1.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockNestedTargetElement2.style.display).toBe('');
    expect(mockNestedTargetElement2.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
  });

  test('should handle a large number of conditions efficiently', () => {
    const largeNumberOfConditions = Array(1000).fill().map((_, i) => ({ Target: `target${i}`, Value: `value${i}` }));
    const mockSelectElement = {
      getAttribute: jest.fn().mockReturnValue(JSON.stringify(largeNumberOfConditions)),
      value: 'value999',
      closest: jest.fn().mockReturnValue(mockFieldset),
    };

    const mockTargetElements = largeNumberOfConditions.map(({ Target }) => ({
      style: { display: 'none' },
      setAttribute: jest.fn(),
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([]),
    }));

    mockFieldset.querySelectorAll.mockImplementation((selector) => {
      const index = parseInt(selector.replace('.target', ''));
      return [mockTargetElements[index]];
    });

    const startTime = performance.now();
    handleRevealConditions(mockSelectElement);
    const endTime = performance.now();

    const executionTime = endTime - startTime;
    console.log(`Execution time: ${executionTime} ms`);

    expect(executionTime).toBeLessThan(500);

    expect(mockTargetElements[999].style.display).toBe('');
    expect(mockTargetElements[999].removeAttribute).toHaveBeenCalledWith('data-reveal-target');

    expect(mockTargetElements[0].style.display).toBe('none');
    expect(mockTargetElements[0].setAttribute).toHaveBeenCalledWith('data-reveal-target', '');
  });
});