import { handleInputChange } from './helpers';

// Mock DOM methods and properties
document.querySelectorAll = jest.fn();

describe('handleInputChange', () => {
  beforeEach(() => {
    jest.clearAllMocks();
  });

  test('should do nothing when isDynamicSetter is false', () => {
    const mockEvent = {
      target: {
        dataset: {
          isDynamicSetter: 'false',
          targetClass: '.target',
          placeholder: 'placeholder'
        },
        value: 'test value'
      }
    };

    handleInputChange(mockEvent);

    expect(document.querySelectorAll).not.toHaveBeenCalled();
  });

  test('should update elements when isDynamicSetter is true', () => {
    const mockInput = { value: 'old-value', tagName: 'INPUT' };
    const mockTextarea = { value: 'old-value', tagName: 'TEXTAREA' };
    const mockDiv = { innerText: 'old-value', tagName: 'DIV' };
    const mockSpan = { textContent: 'old-value', tagName: 'SPAN' };

    document.querySelectorAll.mockReturnValue([mockInput, mockTextarea, mockDiv, mockSpan]);

    const mockEvent = {
      target: {
        dataset: {
          isDynamicSetter: 'true',
          targetClass: '.target',
          placeholder: 'old-value'
        },
        value: 'New Value'
      }
    };

    handleInputChange(mockEvent);

    expect(document.querySelectorAll).toHaveBeenCalledWith('.target');
    expect(mockInput.value).toBe('new-value');
    expect(mockTextarea.value).toBe('new-value');
    expect(mockDiv.innerText).toBe('new-value');
    expect(mockSpan.textContent).toBe('new-value');
    expect(mockEvent.target.dataset.placeholder).toBe('new-value');
  });

  test('should replace all occurrences of the placeholder', () => {
    const mockInput = { value: 'old-value is here and old-value is there', tagName: 'INPUT' };
    document.querySelectorAll.mockReturnValue([mockInput]);

    const mockEvent = {
      target: {
        dataset: {
          isDynamicSetter: 'true',
          targetClass: '.target',
          placeholder: 'old-value'
        },
        value: 'New Value'
      }
    };

    handleInputChange(mockEvent);

    expect(mockInput.value).toBe('new-value is here and new-value is there');
  });

  test('should handle no matching elements', () => {
    document.querySelectorAll.mockReturnValue([]);

    const mockEvent = {
      target: {
        dataset: {
          isDynamicSetter: 'true',
          targetClass: '.non-existent',
          placeholder: 'old-value'
        },
        value: 'New Value'
      }
    };

    handleInputChange(mockEvent);

    expect(document.querySelectorAll).toHaveBeenCalledWith('.non-existent');
    expect(mockEvent.target.dataset.placeholder).toBe('new-value');
  });

  test('should handle lowercase input correctly', () => {
    const mockInput = { value: 'OLD-VALUE', tagName: 'INPUT' };
    document.querySelectorAll.mockReturnValue([mockInput]);

    const mockEvent = {
      target: {
        dataset: {
          isDynamicSetter: 'true',
          targetClass: '.target',
          placeholder: 'OLD-VALUE'
        },
        value: 'new value'
      }
    };

    handleInputChange(mockEvent);

    expect(mockInput.value).toBe('new-value');
    expect(mockEvent.target.dataset.placeholder).toBe('new-value');
  });
});