import { removeItem } from './removeitem';

describe('removeItem', () => {
  let mockEl, mockElement, mockParentOfParent, mockSelectorInput, mockMainInput;

  beforeEach(() => {
    jest.clearAllMocks();

    mockEl = {
      parentNode: {
        dataset: { input: '.selector-input' },
        remove: jest.fn(),
        parentNode: {},
        innerText: 'xTestValue',
        querySelector: jest.fn()
      }
    };

    mockElement = mockEl.parentNode;
    mockParentOfParent = mockElement.parentNode;

    mockSelectorInput = { value: 'TestValue,AnotherValue' };
    mockMainInput = { value: 'TestValue,AnotherValue,ThirdValue' };

    mockParentOfParent.querySelector = jest.fn();
    mockParentOfParent.parentNode = {
      querySelector: jest.fn()
    };
  });

  test('should remove the item from the DOM', () => {
    removeItem(mockEl);
    expect(mockElement.remove).toHaveBeenCalled();
  });

  test('should update the selector input value', () => {
    mockParentOfParent.querySelector.mockReturnValue(mockSelectorInput);
    removeItem(mockEl);
    expect(mockParentOfParent.querySelector).toHaveBeenCalledWith('.selector-input');
    expect(mockSelectorInput.value).toBe('AnotherValue');
  });

  test('should update the main input value', () => {
    mockParentOfParent.querySelector.mockReturnValue(mockSelectorInput);
    mockParentOfParent.parentNode.querySelector.mockReturnValue(mockMainInput);
    removeItem(mockEl);
    expect(mockParentOfParent.parentNode.querySelector).toHaveBeenCalledWith('input[type="hidden"]');
    expect(mockMainInput.value).toBe('AnotherValue,ThirdValue');
  });

  test('should not throw error when selector input is not found', () => {
    mockParentOfParent.querySelector.mockReturnValue(null);
    expect(() => removeItem(mockEl)).not.toThrow();
  });

  test('should not throw error when main input is not found', () => {
    mockParentOfParent.querySelector.mockReturnValue(mockSelectorInput);
    mockParentOfParent.parentNode.querySelector.mockReturnValue(null);
    expect(() => removeItem(mockEl)).not.toThrow();
  });

  test('should remove the only value from inputs', () => {
    mockSelectorInput.value = 'TestValue';
    mockMainInput.value = 'TestValue';
    mockParentOfParent.querySelector.mockReturnValue(mockSelectorInput);
    mockParentOfParent.parentNode.querySelector.mockReturnValue(mockMainInput);
    removeItem(mockEl);
    expect(mockSelectorInput.value).toBe('');
    expect(mockMainInput.value).toBe('');
  });

  test('should not change input values when removing non-existent value', () => {
    mockElement.innerText = 'xNonExistentValue';
    mockParentOfParent.querySelector.mockReturnValue(mockSelectorInput);
    mockParentOfParent.parentNode.querySelector.mockReturnValue(mockMainInput);
    removeItem(mockEl);
    expect(mockSelectorInput.value).toBe('TestValue,AnotherValue');
    expect(mockMainInput.value).toBe('TestValue,AnotherValue,ThirdValue');
  });

  test('should correctly extract value by removing first character of innerText', () => {
    mockElement.innerText = 'xSpecialValue';
    mockSelectorInput.value = 'SpecialValue,OtherValue';
    mockMainInput.value = 'SpecialValue,OtherValue,ThirdValue';
    mockParentOfParent.querySelector.mockReturnValue(mockSelectorInput);
    mockParentOfParent.parentNode.querySelector.mockReturnValue(mockMainInput);
    removeItem(mockEl);
    expect(mockSelectorInput.value).toBe('OtherValue');
    expect(mockMainInput.value).toBe('OtherValue,ThirdValue');
  });
});