import { setupRevealConditions } from './reveal';

// Mock DOM methods and properties
document.querySelector = jest.fn();
console.log = jest.fn();
console.warn = jest.fn();

describe('setupRevealConditions', () => {
  let mockSelect;
  let mockTarget;

  beforeEach(() => {
    jest.clearAllMocks();

    mockTarget = {
      style: { display: 'none' },
      removeAttribute: jest.fn(),
      querySelectorAll: jest.fn().mockReturnValue([]) // Add querySelectorAll to mockTarget
    };

    // Update mock fieldset to include querySelectorAll
    const mockFieldset = {
      querySelectorAll: jest.fn().mockReturnValue([mockTarget])
    };

    mockSelect = {
      dataset: {
        revealConditions: JSON.stringify([
          { Value: 'value1', Target: 'target1' },
          { Value: 'value2', Target: 'target2' }
        ])
      },
      value: '',
      closest: jest.fn().mockReturnValue(mockFieldset)
    };
  });

  test('should show target element when matching condition is found', () => {
    mockSelect.value = 'value1';
    const fieldset = mockSelect.closest();

    const update = setupRevealConditions(mockSelect);

    expect(fieldset.querySelectorAll).toHaveBeenCalledWith('.target1');
    expect(mockTarget.style.display).toBe('');
    expect(mockTarget.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockTarget.querySelectorAll).toHaveBeenCalledWith('select[data-reveal-conditions]');
   });

  test('should warn when target element is not found', () => {
    mockSelect.value = 'value1';
    const fieldset = mockSelect.closest();
    fieldset.querySelectorAll.mockReturnValue([]);

    const update = setupRevealConditions(mockSelect);

    expect(fieldset.querySelectorAll).toHaveBeenCalledWith('.target1');
  });

  test('should do nothing when no matching condition is found', () => {
    mockSelect.value = 'value3';

    const update = setupRevealConditions(mockSelect);

    expect(document.querySelector).not.toHaveBeenCalled();
    expect(console.log).not.toHaveBeenCalled();
    expect(console.warn).not.toHaveBeenCalled();
  });

  test('should call update function initially', () => {
    mockSelect.value = 'value2';
    const fieldset = mockSelect.closest();

    setupRevealConditions(mockSelect);

    expect(fieldset.querySelectorAll).toHaveBeenCalledWith('.target2');
    expect(mockTarget.style.display).toBe('');
    expect(mockTarget.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockTarget.querySelectorAll).toHaveBeenCalledWith('select[data-reveal-conditions]');
});

  test('should return update function that can be called later', () => {
    mockSelect.value = 'value1';
    const fieldset = mockSelect.closest();
    const update = setupRevealConditions(mockSelect);

    // Clear mocks to check the next update call
    jest.clearAllMocks();

    mockSelect.value = 'value2';
    update();

    expect(fieldset.querySelectorAll).toHaveBeenCalledWith('.target2');
    expect(mockTarget.style.display).toBe('');
    expect(mockTarget.removeAttribute).toHaveBeenCalledWith('data-reveal-target');
    expect(mockTarget.querySelectorAll).toHaveBeenCalledWith('select[data-reveal-conditions]');
  });
});
