(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as anonymous module.
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node / CommonJS
        module.exports = factory(require('jquery'));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    'use strict';

    var NAMESPACE = 'qor.chooser';
    var EVENT_ENABLE = 'enable.' + NAMESPACE;
    var EVENT_DISABLE = 'disable.' + NAMESPACE;

    function QorChooser(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorChooser.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorChooser.prototype = {
        constructor: QorChooser,

        getSelect2AjaxDynamicURL: function() {
            let url = this.$element.attr('data-remote-url');
            let name = this.$element.attr('name');
            let match = name.match(/\[(\d+)\]/);
            let targetClass = this.$element.attr('data-dynamic-remote-url-target');
            
            if (targetClass) {
                // Split target classes and handle multiple placeholders
                let targetClasses = targetClass.split(',').map(c => c.trim());
                let placeholders = ['placeholder', 'placeholder2', 'placeholder3']; // Support up to 3 placeholders
                
                targetClasses.forEach((targetClass, index) => {
                    let targetSelector = targetClass;
                    if (match) {
                        let arrayIndex = match[1];
                        // try to find the target element
                        targetSelector = `${targetClass}[name*="[${arrayIndex}]"]`;
                        let targetElement = document.querySelector(targetSelector);
                        
                        // if not found, try without the index
                        if (!targetElement) {
                            targetSelector = targetClass;
                            targetElement = document.querySelector(targetSelector);
                        }
                        
                        
                        if (targetElement) {
                            url = url.replace(placeholders[index], targetElement.value);
                        }
                    } else {
                        // if no index, try to find the target element without the index
                        let targetElement = document.querySelector(targetSelector);
                        
                        if (targetElement) {
                            url = url.replace(placeholders[index], targetElement.value);
                        }
                    }
                });
            }
            return url;
        },

        init: function() {
            let $this = this.$element,
                select2Data = $this.data(),
                resetSelect2Width,
                option = {
                    minimumResultsForSearch: 8,
                    dropdownParent: $this.parent()
                };

            $this.find('option').each(function() {
                let crypto = window.crypto || window.msCrypto;
                // Improve the randomness of this
                $(this).attr('data-select2-id', Array.from(crypto.getRandomValues(new Uint8Array(16)))
                    .map(b => b.toString(16).padStart(2, '0'))
                    .join(''));
            });        

            let remoteImage = select2Data.remoteImage;

            if (select2Data.remoteData) {
                option.ajax = $.fn.select2.ajaxCommonOptions(select2Data);
                option.ajax.transport = (params, success, failure) => {
                    let url = this.getSelect2AjaxDynamicURL();
                    let $request = $.ajax(url, params);
                    $request.then(success);
                    $request.fail(failure);
                    return $request;
                };

                option.templateResult = function(data) {
                    let tmpl = $this.parents('.qor-field').find('[name="select2-result-template"]');
                    return $.fn.select2.ajaxFormatResult(data, tmpl, remoteImage);
                };

                option.templateSelection = function(data) {
                    if (data.loading) return data.text;
                    let tmpl = $this.parents('.qor-field').find('[name="select2-selection-template"]');
                    return $.fn.select2.ajaxFormatResult(data, tmpl, remoteImage);
                };
            }

            $this
                .on('select2:select', function(evt) {
                    $(evt.target).attr('chooser-selected', 'true').trigger('change');
                })
                .on('select2:unselect', function(evt) {
                    $(evt.target).attr('chooser-selected', '').trigger('change');
                })
                .on("change", function(evt) {
                    let $input = $(evt.target);
                    let isDynamicSetter = $input.data('isDynamicSetter') === true;
                    let setterClass = $input.data('setterClass');

                    $input.next().find('.select2-selection').removeClass('error-highlight');

                    if (setterClass) {
                        let name = $input.attr('name');
                        let match = name.match(/\[(\d+)\]/);
                        let targetSelector = setterClass;
                        if (match) {
                            let index = match[1];
                            targetSelector = `${setterClass}[name*="[${index}]"]`;
                        }
                        let targetElement = $(targetSelector);
                        if (targetElement){
                            let selection = targetElement.next().find('.select2-selection__rendered');
                            let selectionParent = selection.parent();
                            selection.remove();
                            selectionParent.prepend(`
                                <span class="select2-selection__rendered" id="select2--container" role="textbox" aria-readonly="true">
                                    <span class="select2-selection__placeholder">Select an Option</span>
                                </span>
                            `);
                        }
                    }

                    if (!isDynamicSetter) return;
                    
                    let targetClass = $input.data('targetClass');
                    let placeholder = $input.data('targetPlaceholder'); 
                    let elementsToUpdate = document.querySelectorAll(targetClass);
                    let formattedValue = $input.val();
            
                    elementsToUpdate.forEach(element => {
                        const $element = $(element);
                        if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
                            $element.val($element.val().replace(new RegExp(placeholder, 'g'), formattedValue));
                        } else if (element.tagName === 'DIV') {
                            $element.text($element.text().replace(new RegExp(placeholder, 'g'), formattedValue));
                            console.log("updated element", $element);
                        } else {
                            $element.text($element.text().replace(new RegExp(placeholder, 'g'), formattedValue));
                        }
                    });

                    $input.data('targetPlaceholder', formattedValue);
                });

            $this.select2(option);

            // reset select2 container width
            this.resetSelect2Width();
            resetSelect2Width = window._.debounce(this.resetSelect2Width.bind(this), 300);
            $(window).resize(resetSelect2Width);

            if ($this.val()) {
                $this.attr('chooser-selected', 'true');
            }
        },

        resetSelect2Width: function() {
            var $container,
                select2 = this.$element.data().select2;
            if (select2 && select2.$container) {
                $container = select2.$container;
                $container.width($container.parent().width());
            }
        },

        destroy: function() {
            this.$element.select2('destroy').removeData(NAMESPACE);
        }
    };

    QorChooser.DEFAULTS = {};

    QorChooser.plugin = function(options) {
        return this.each(function() {
            var $this = $(this);
            var data = $this.data(NAMESPACE);
            var fn;

            if (!data) {
                if (/destroy/.test(options)) {
                    return;
                }

                $this.data(NAMESPACE, (data = new QorChooser(this, options)));
            }

            if (typeof options === 'string' && $.isFunction((fn = data[options]))) {
                fn.apply(data);
            }
        });
    };

    $(function() {
        var selector = 'select[data-toggle="qor.chooser"]';

        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorChooser.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorChooser.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    $.fn.qorChooser = QorChooser.plugin;
    return QorChooser;
});
