/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the jQuery Select2 plugin
$.fn.select2 = function(options) {
    return this.each(function() {
        $(this).data('select2-options', options);
        
        // Mock select2 methods
        this.select2 = {
            open: jest.fn(),
            close: jest.fn(),
            destroy: jest.fn(),
        };

        // Call the ready callback if it exists
        if (options && options.ready) {
            options.ready.call(this);
        }
    });
};

$.fn.select2.ajaxCommonOptions = jest.fn(() => ({
    ajax: {
        dataType: 'json',
        delay: 250,
        data: jest.fn(),
        processResults: jest.fn(),
        cache: true
    }
}));

// Mock jQuery's ajax method
$.ajax = jest.fn(() => Promise.resolve({ data: [{ id: 1, text: 'Option 1' }] }));

// Mock lodash debounce
window._ = {
    debounce: jest.fn(fn => fn)
};

const QorChooser = require('./qor-chooser.js');

describe('QorChooser', () => {
    let element;
    let qorChooserInstance;

    beforeEach(() => {
        // Set up your HTML fixture for testing
        document.body.innerHTML = `
            <div class="qor-field">
                <select data-toggle="qor.chooser" data-remote-data="true" data-remote-url="/api/options">
                    <option value="">Select an option</option>
                    <option value="1">Option 1</option>
                    <option value="2">Option 2</option>
                </select>
                <script name="select2-result-template" type="x-tmpl-mustache">
                    <div>{{text}}</div>
                </script>
                <script name="select2-selection-template" type="x-tmpl-mustache">
                    <div>{{text}}</div>
                </script>
            </div>
        `;
    
        element = $('select[data-toggle="qor.chooser"]');
    
        // Initialize QorChooser
        $.fn.qorChooser = QorChooser.plugin;
        element.qorChooser();
        qorChooserInstance = element.data('qor.chooser');
    });

    afterEach(() => {
        // Clean up the DOM
        document.body.innerHTML = '';
        jest.restoreAllMocks();
    });

    test('should initialize QorChooser correctly', () => {
        expect(qorChooserInstance).toBeDefined();
        expect(qorChooserInstance.$element[0]).toBe(element[0]);
        expect(qorChooserInstance.options).toEqual({});
    });

    test('should set up Select2 with correct options', () => {
        const select2Options = element.data('select2-options');
        expect(select2Options).toBeDefined();
        expect(select2Options.minimumResultsForSearch).toBe(8);
        expect(select2Options.dropdownParent.is(element.parent())).toBe(true);
    });

    test('should handle remote data correctly', () => {
        const select2Options = element.data('select2-options');
        expect(select2Options.ajax).toBeDefined();
        expect(typeof select2Options.templateResult).toBe('function');
        expect(typeof select2Options.templateSelection).toBe('function');
    });

    test('should handle select and unselect events', () => {
        const selectEvent = $.Event('select2:select');
        element.trigger(selectEvent);
        expect(element.attr('chooser-selected')).toBe('true');

        const unselectEvent = $.Event('select2:unselect');
        element.trigger(unselectEvent);
        expect(element.attr('chooser-selected')).toBe('');
    });

    test('should handle dynamic URL generation', () => {
        element.attr('data-dynamic-remote-url-target', '.target-input');
        element.attr('name', 'chooser[0]');
        
        // Add a target input to the DOM
        $('<input class="target-input" name="target[0]" value="dynamic-value">').appendTo('body');

        const url = qorChooserInstance.getSelect2AjaxDynamicURL();
        expect(url).toBe('/api/options'.replace('placeholder', 'dynamic-value'));
    });

    test('should handle multiple parameters in dynamic URL generation', () => {
        element.attr('data-dynamic-remote-url-target', '.target-input-1,.target-input-2');
        element.attr('name', 'chooser[0]');
        
        // Add target inputs to the DOM with different values
        $('<input class="target-input-1" name="target1[0]" value="value1">').appendTo('body');
        $('<input class="target-input-2" name="target2[0]" value="value2">').appendTo('body');
        
        const url = qorChooserInstance.getSelect2AjaxDynamicURL();
        expect(url).toBe('/api/options'.replace('placeholder', 'value1'));
        expect(url).toBe('/api/options'.replace('placeholder2', 'value2'));
    });

    test('should handle change event for dynamic setter', () => {
        element.data('isDynamicSetter', true);
        element.data('targetClass', '.target-element');
        element.data('targetPlaceholder', 'placeholder');
    
        // Add target elements to the DOM
        $('<input class="target-element" value="Old placeholder value">').appendTo('body');
        $('<div class="target-element">Old placeholder text</div>').appendTo('body');
    
        // Mock the updateDynamicSetter function
        qorChooserInstance.updateDynamicSetter = jest.fn((value) => {
            $('.target-element').val(`Old ${value} value`);
            $('.target-element').text(`Old ${value} text`);
        });
    
        element.val('New Value').trigger('change');
    
        expect($('input.target-element').val()).toBe('Old null value');
        expect($('div.target-element').text()).toBe('Old null text');
    });
    
    test('should reset Select2 width on window resize', () => {
        const resizeSpy = jest.spyOn(qorChooserInstance, 'resetSelect2Width');
        qorChooserInstance.init(); // Ensure the resize event is bound
        $(window).trigger('resize');
        expect(resizeSpy).toHaveBeenCalled();
    });
    describe('setter class functionality', () => {
        beforeEach(() => {
            // Reset DOM for each test
            document.body.innerHTML = `
                <select class="source-select" data-setter-class=".target-select" name="source[0]"></select>
                <select class="target-select" name="target[0]">
                    <option value="1">Option 1</option>
                </select>
                <span class="select2-container">
                    <span class="select2-selection">
                        <span class="select2-selection__rendered">Current Selection</span>
                    </span>
                </span>
            `;
            
            element = $('.source-select');
            $.fn.qorChooser = QorChooser.plugin;
            element.qorChooser();
            qorChooserInstance = element.data('qor.chooser');
        });

        test('should reset target select2 when source changes', () => {
            // Setup
            element.data('setterClass', '.target-select');
            
            // Trigger change event
            element.trigger('change');

            // Get the new rendered content
            const newRenderedContent = $('.target-select')
                .next()
                .find('.select2-selection__rendered')
                .find('.select2-selection__placeholder')
                .text();

            // Assert the placeholder is set correctly
            expect(newRenderedContent).toBe('Select an Option');
        });

        test('should handle array indices in names correctly', () => {
            // Setup with array index in name
            document.body.innerHTML = `
                <select class="source-select" data-setter-class=".target-select" name="source[2]"></select>
                <select class="target-select" name="target[2]">
                    <option value="1">Option 1</option>
                </select>
                <span class="select2-container">
                    <span class="select2-selection">
                        <span class="select2-selection__rendered">Current Selection</span>
                    </span>
                </span>
            `;
            
            element = $('.source-select');
            element.qorChooser();
            element.data('setterClass', '.target-select');
            
            // Trigger change event
            element.trigger('change');

            // Assert the correct target was affected
            const newRenderedContent = $('select[name="target[2]"]')
                .next()
                .find('.select2-selection__rendered')
                .find('.select2-selection__placeholder')
                .text();

            expect(newRenderedContent).toBe('Select an Option');
        });

        test('should not affect other select elements', () => {
            // Setup with multiple selects
            document.body.innerHTML = `
                <select class="source-select" data-setter-class=".target-select" name="source[0]"></select>
                <select class="target-select" name="target[0]"></select>
                <select class="other-select" name="other[0]"></select>
                <span class="select2-container">
                    <span class="select2-selection">
                        <span class="select2-selection__rendered">Should Not Change</span>
                    </span>
                </span>
            `;
            
            element = $('.source-select');
            element.qorChooser();
            element.data('setterClass', '.target-select');
            
            const originalContent = $('.other-select')
                .next()
                .find('.select2-selection__rendered')
                .text();

            // Trigger change event
            element.trigger('change');

            // Assert other select was not affected
            const unchangedContent = $('.other-select')
                .next()
                .find('.select2-selection__rendered')
                .text();

            expect(unchangedContent).toBe(originalContent);
        });
    });
    test('should destroy QorChooser correctly', () => {
        const select2DestroyMock = jest.fn();
        element.data('select2', { destroy: select2DestroyMock });
        
        // Mock the removeData method
        const removeDataMock = jest.fn();
        element.removeData = removeDataMock;
    
        qorChooserInstance.destroy();
    });
});