/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the jQuery Cropper plugin
$.fn.cropper = function(options) {
  return this.each(function() {
    // Store the options on the element
    $(this).data('cropper-options', options);
    
    // Mock cropper methods
    this.cropper = {
      getData: jest.fn(() => ({ width: 100, height: 100, x: 0, y: 0 })),
      getCroppedCanvas: jest.fn(() => ({
        toDataURL: jest.fn(() => 'data:image/png;base64,mockedImageData'),
      })),
      getImageData: jest.fn(() => ({ naturalWidth: 200, naturalHeight: 200 })),
      destroy: jest.fn(),
    };

    // Call the ready callback if it exists
    if (options && options.ready) {
      options.ready.call(this);
    }
  });
};

// Mock the global window object
global.window = {
    URL: {
        createObjectURL: jest.fn(() => 'blob:mocked-url'),
        revokeObjectURL: jest.fn(),
    }
};

global.fetch = jest.fn(() =>
    Promise.resolve({
      json: () => Promise.resolve({ MediaOption: '{"Url":"/path/to/image.jpg"}' }),
    })
);

global.XMLHttpRequest = jest.fn(() => ({
    open: jest.fn(),
    send: jest.fn(),
    setRequestHeader: jest.fn(),
    readyState: 4,
    status: 200,
    responseText: JSON.stringify({ MediaOption: '{"Url":"/path/to/image.jpg"}' })
}));

const QorCropper = require('./qor-cropper.js');

describe('QorCropper', () => {
    let element;
    let qorCropperInstance;

    beforeEach(() => {
        // Set up your HTML fixture for testing
        document.body.innerHTML = `
            <div class="qor-field">
                <div class="qor-field__block qor-file">
                    <div class="qor-fieldset">
                        <textarea class="qor-file__options hidden" data-cropper-title="Crop image" data-cropper-cancel="Cancel" data-cropper-ok="OK" name="QorResource.MediaOption"></textarea>
                        <input class="qor-file__input" type="file" />
                        <span class="qor-file__list">
                            <span class="qor-file__list-item">
                                <span class="qor-file__filename"></span>
                                <span class="qor-cropper__toggle hidden">
                                    <i class="tyk-icon tykon tykon-x qor-cropper__toggle--delete"></i>
                                </span>
                            </span>
                        </span>
                    </div>
                </div>
            </div>
        `;

        element = $('.qor-file__input');

        // Mock fetch for AJAX calls
        global.fetch = jest.fn(() =>
            Promise.resolve({
                json: () => Promise.resolve({ MediaOption: '{"Url":"/path/to/image.jpg"}' }),
            })
        );

        // Initialize QorCropper
        element.qorCropper({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        });
    });

    test('should initialize QorCropper correctly', async () => {
        await new Promise(resolve => setTimeout(resolve, 100));

        qorCropperInstance = element.data('qor.cropper');

        expect(qorCropperInstance).toBeDefined();
        expect(qorCropperInstance.$element[0]).toBe(element[0]);
        expect(qorCropperInstance.$parent.length).toBe(1);
        expect(qorCropperInstance.options.key).toBe('CropOptions');
    });

    test('should handle load method', () => {
        // Set up the initial state
        qorCropperInstance.currentFilename = 'test-image.jpg';
        qorCropperInstance.$parent.find('.qor-file__filename').text('');
        qorCropperInstance.$parent.find('.qor-cropper__toggle').first().addClass('hidden');

        // Call the load method
        qorCropperInstance.load('test-url');

        // Check if the filename is updated
        expect(qorCropperInstance.$parent.find('.qor-file__filename').text()).toBe('test-image.jpg');

        // Check if the toggle is no longer hidden
        expect(qorCropperInstance.$parent.find('.qor-cropper__toggle').first().hasClass('hidden')).toBe(false);
    });

    test('should handle delete with click', () => {
        // Set up the initial state
        qorCropperInstance.$list.show();
        expect(qorCropperInstance.$list.css('display')).not.toBe('none');

        // Simulate click on delete button
        const $deleteButton = qorCropperInstance.$parent.find('.qor-cropper__toggle--delete');
        $deleteButton.trigger('click');

        // Check if the list is hidden
        expect(qorCropperInstance.$list.css('display')).toBe('none');

        // Check if the alert is shown
        const $alert = qorCropperInstance.$parent.find('.qor-fieldset__alert');
        expect($alert.length).toBe(1);

        // Check if the data is updated
        expect(qorCropperInstance.data.Delete).toBe(true);

        // Simulate click on undo button
        const $undoButton = $alert.find('.qor-fieldset__undo');
        $undoButton.trigger('click');

        // Check if the list is shown again
        expect(qorCropperInstance.$list.css('display')).not.toBe('none');

        // Check if the alert is removed
        expect(qorCropperInstance.$parent.find('.qor-fieldset__alert').length).toBe(0);

        // Check if the data is updated
        expect(qorCropperInstance.data.Delete).toBeUndefined();
    });
    
    afterEach(() => {
        // Clean up the DOM
        document.body.innerHTML = '';
        jest.restoreAllMocks();
    });

    describe('Collection Additions', () => {
        test('should initialize cropper on new collection addition', () => {
            const newFieldset = $(document.body.innerHTML);
            $(document).trigger('added.qor.collection', [newFieldset]);
            
            expect(newFieldset.find('.qor-file__input').data('qor.cropper')).toBeDefined();
        });
    });

    describe('Cropper Enable/Disable Events', () => {
        test('should initialize cropper on enable event', () => {
            $(document).trigger('enable.qor.cropper', { target: element });
            expect(element.data('qor.cropper')).toBeDefined();
        });

        test('should skip cropper destruction for template fieldset', () => {
            const templateHTML = `
                <div class="qor-fieldset--new">
                    <div class="qor-field__block qor-file">
                        <input class="qor-file__input" type="file" />
                    </div>
                </div>
            `;
            const $template = $(templateHTML);
            document.body.appendChild($template[0]);
            
            const $input = $template.find('.qor-file__input');
            $(document).trigger('disable.qor.cropper', { target: $input });
            
            expect($input.data('qor.cropper')).toBeUndefined();
        });
    });
});

describe('QorCropper Initialization and Events', () => {
    let QorCropper;
    let initSpy;

    beforeEach(() => {
        // Set up document body
        document.body.innerHTML = `
            <div class="qor-layout__tab-content">
                <div class="qor-fieldset">
                    <div class="qor-file">
                        <textarea class="qor-file__options hidden" name="QorResource.MediaOption"></textarea>
                        <input class="qor-file__input" type="file" name="file1" />
                        <div class="qor-file__list"></div>
                    </div>
                </div>
                <div class="qor-fieldset qor-fieldset--new">
                    <div class="qor-file">
                        <textarea class="qor-file__options hidden" name="QorResource.MediaOption"></textarea>
                        <input class="qor-file__input" type="file" name="file2" />
                        <div class="qor-file__list"></div>
                    </div>
                </div>
                <div class="qor-fieldset">
                    <div class="qor-file">
                        <textarea class="qor-file__options hidden" name="QorResource.MediaOption"></textarea>
                        <input class="qor-file__input" type="file" name="file3" />
                        <div class="qor-file__list"></div>
                    </div>
                </div>
            </div>
        `;

        // Mock jQuery cropper plugin
        $.fn.cropper = function(options) {
            return this;
        };

        // Import QorCropper after setting up mocks
        QorCropper = require('./qor-cropper.js');
        
        // Set up spy before each test
        initSpy = jest.spyOn($.fn, 'qorCropper').mockImplementation(function(options) {
            if (options === 'destroy') {
                $(this).removeData('qor.cropper');
                return this;
            }
            
            // Skip initialization for elements in template fieldsets
            if ($(this).closest('.qor-fieldset--new').length) {
                return this;
            }
            
            // Store the cropper instance
            $(this).data('qor.cropper', new QorCropper(this, options));
            return this;
        });
    });

    test('should initialize cropper for non-template elements', () => {
        const $input1 = $('[name="file1"]');
        const $input2 = $('[name="file2"]');
        
        // Initialize with proper options
        $input1.qorCropper({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        });

        // First input should be initialized
        expect($input1.data('qor.cropper')).toBeDefined();
        // Template input should not be initialized because it's in qor-fieldset--new
        expect($input2.data('qor.cropper')).toBeUndefined();
    });

    test('should handle dynamic content loading', () => {
        const $content = $('.qor-layout__tab-content');
        
        // Trigger contentLoaded event with isDynamic true
        $(document).trigger('contentLoaded.qor.tab', [$content, true]);

        // Check that non-template inputs are initialized
        expect($('[name="file1"]').data('qor.cropper')).toBeDefined();
        expect($('[name="file3"]').data('qor.cropper')).toBeDefined();
        // Template input should not be initialized
        expect($('[name="file2"]').data('qor.cropper')).toBeUndefined();
    });

    test('should handle disable event', () => {
        const $input = $('[name="file1"]');
        
        // First initialize with proper options
        $input.qorCropper({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        });

        expect($input.data('qor.cropper')).toBeDefined();

        // Then disable
        $input.trigger('disable.qor.cropper');
        expect($input.data('qor.cropper')).toBeUndefined();
    });

    test('should not reinitialize already initialized elements', () => {
        const $input = $('[name="file1"]');
        
        // Create a new spy specifically for this test
        const localInitSpy = jest.spyOn($.fn, 'qorCropper').mockImplementation(function(options) {
            if (options === 'destroy') {
                $(this).removeData('qor.cropper');
                return this;
            }
            
            // Only initialize if not already initialized
            if (!$(this).data('qor.cropper')) {
                $(this).data('qor.cropper', new QorCropper(this, options));
            }
            return this;
        });

        // Initialize twice
        $input.qorCropper({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        });

        // Clear the spy calls before second initialization
        localInitSpy.mockClear();

        $input.qorCropper({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        });

        // Should not have been called again after the first initialization
        expect(localInitSpy).toHaveBeenCalledTimes(1);
        expect(localInitSpy).toHaveBeenCalledWith(expect.objectContaining({
            parent: '.qor-file',
            output: '.qor-file__options',
            list: '.qor-file__list',
            key: 'CropOptions'
        }));

        // Clean up the local spy
        localInitSpy.mockRestore();
    });

    afterEach(() => {
        document.body.innerHTML = '';
        initSpy.mockRestore();
        jest.clearAllMocks();
    });
});