(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as anonymous module.
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node / CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    'use strict';

    const NAMESPACE = 'qor.metadata';
    const EVENT_ENABLE = 'enable.' + NAMESPACE;
    const EVENT_DISABLE = 'disable.' + NAMESPACE;
    const EVENT_CLICK = 'click.' + NAMESPACE;
    
    const CLASS_CONTAINER = '.metadata-container';
    const CLASS_FIELDS = '.metadata-fields';
    const CLASS_ROW = '.metadata-row';
    const CLASS_INDEX = '.metadata-index';
    const CLASS_ADD = '.metadata-add';
    const CLASS_DELETE = '.metadata-delete';
    const CLASS_TEMPLATE = '#metadata-template';
    const CLASS_LABEL_WRAPPER = '.input-label-wrapper';

    function QorMetadata(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorMetadata.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorMetadata.prototype = {
        constructor: QorMetadata,

        init: function() {
            this.$container = this.$element;
            this.$fields = this.$container.find(CLASS_FIELDS);
            this.$template = this.$container.find(CLASS_TEMPLATE);
            this.bind();
            this.removeLabels();
        },

        bind: function() {
            this.$container
                .on(EVENT_CLICK, CLASS_ADD, this.addRow.bind(this))
                .on(EVENT_CLICK, CLASS_DELETE, this.deleteRow.bind(this));
        },

        unbind: function() {
            this.$container.off(EVENT_CLICK);
        },

        removeLabels: function() {
            this.$container
                .find(CLASS_INDEX + '[data-remove-labels="true"]')
                .find(CLASS_LABEL_WRAPPER)
                .remove();
        },

        addRow: function() {
            const currentCount = this.$fields.find(CLASS_ROW).length;
            const $clone = $(this.$template.html());
            
            $clone.find(CLASS_LABEL_WRAPPER).remove();
            $clone.find(CLASS_INDEX).attr('data-index', currentCount);
            
            $clone.find('input, select, textarea').each((i, input) => {
                const name = $(input).attr('name');
                if (name) {
                    $(input).attr('name', name.replace('[0]', `[${currentCount}]`));
                }
            });
            
            this.$fields.append($clone);
        },

        deleteRow: function(e) {
            const $rows = this.$fields.find(CLASS_ROW);
            if ($rows.length <= 1) return;

            const $deletedRow = $(e.target).closest(CLASS_ROW);
            $deletedRow.remove();

            // Reindex remaining rows
            this.$fields.find(CLASS_ROW).each((index, row) => {
                const $row = $(row);
                $row.find(CLASS_INDEX).attr('data-index', index);
                
                $row.find('input, select, textarea').each((i, input) => {
                    const name = $(input).attr('name');
                    if (name) {
                        $(input).attr('name', name.replace(/\[\d+\]/, `[${index}]`));
                    }
                });
            });
        }
    };

    QorMetadata.DEFAULTS = {};

    QorMetadata.plugin = function(options) {
        return this.each(function() {
            const $this = $(this);
            let data = $this.data(NAMESPACE);

            if (!data) {
                if (/destroy/.test(options)) return;
                $this.data(NAMESPACE, (data = new QorMetadata(this, options)));
            }

            if (typeof options === 'string' && $.isFunction(data[options])) {
                data[options].apply(data);
            }
        });
    };

    $(function() {
        const selector = CLASS_CONTAINER;
        
        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorMetadata.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorMetadata.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    $.fn.qorMetadata = QorMetadata.plugin;

    return QorMetadata;
});