/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the module loading environment
global.jQuery = $;
global.define = undefined;
global.exports = undefined;
global.module = undefined;

// Now require the module
require('./qor-metadata');

describe('QorMetadata', () => {
    let element;
    let qorMetadataInstance;

    beforeEach(() => {
        document.body.innerHTML = `
            <div class="metadata-container">
                <div class="metadata-fields">
                    <div class="metadata-row">
                        <div class="metadata-index" data-index="0">
                            <div class="input-label-wrapper">Label 1</div>
                            <input type="text" name="field[0].name" value="test1">
                        </div>
                        <button class="metadata-add" type="button">+</button>
                    </div>
                </div>
                <template id="metadata-template">
                    <div class="metadata-row margin-top-n60">
                        <div class="metadata-index" data-index="0">
                            <div class="input-label-wrapper">Label New</div>
                            <input type="text" name="field[0].name" value="">
                        </div>
                        <button class="metadata-delete" type="button"></button>
                    </div>
                </template>
            </div>
        `;

        element = $('.metadata-container');
        element.qorMetadata();
        qorMetadataInstance = element.data('qor.metadata');
    });

    afterEach(() => {
        element.qorMetadata('destroy');
        document.body.innerHTML = '';
    });

    test('should initialize QorMetadata correctly', () => {
        expect(qorMetadataInstance).toBeDefined();
        expect(qorMetadataInstance.$element[0]).toBe(element[0]);
        expect(qorMetadataInstance.$fields.length).toBe(1);
        expect(qorMetadataInstance.$template.length).toBe(1);
    });

    test('should add new row when clicking add button', () => {
        // Initial state
        expect(element.find('.metadata-row').length).toBe(1);

        // Trigger add button click
        element.find('.metadata-add').trigger('click');

        // Check if new row was added
        expect(element.find('.metadata-row').length).toBe(2);
        
        // Check if new row has correct index
        const newRow = element.find('.metadata-row').last();
        expect(newRow.find('.metadata-index').attr('data-index')).toBe('1');
        
        // Check if input name was updated correctly
        expect(newRow.find('input').attr('name')).toBe('field[1].name');
    });

    test('should remove labels from rows after first one', () => {
        // Add a second row
        element.find('.metadata-add').trigger('click');

        // Check if labels were removed from second row
        const secondRow = element.find('.metadata-row').last();
        expect(secondRow.find('.input-label-wrapper').length).toBe(0);
    });

    test('should delete row and reindex remaining rows', () => {
        // Add two more rows
        element.find('.metadata-add').trigger('click');
        element.find('.metadata-add').trigger('click');
        expect(element.find('.metadata-row').length).toBe(3);

        // Delete middle row
        element.find('.metadata-row').eq(1).find('.metadata-delete').trigger('click');

        // Check if row was deleted
        expect(element.find('.metadata-row').length).toBe(2);

        // Check if remaining rows were reindexed
        const rows = element.find('.metadata-row');
        expect(rows.eq(0).find('.metadata-index').attr('data-index')).toBe('0');
        expect(rows.eq(1).find('.metadata-index').attr('data-index')).toBe('1');
        expect(rows.eq(1).find('input').attr('name')).toBe('field[1].name');
    });

    test('should not delete row if only one remains', () => {
        // Try to delete the only row
        element.find('.metadata-delete').trigger('click');

        // Check if row still exists
        expect(element.find('.metadata-row').length).toBe(1);
    });

    test('should unbind events on destroy', () => {
        const $container = qorMetadataInstance.$container;
        
        // Spy on off method
        const offSpy = jest.spyOn($.fn, 'off');
        
        // Call destroy
        qorMetadataInstance.unbind();
        
        // Verify event was unbound
        expect(offSpy).toHaveBeenCalledWith('click.qor.metadata');
        
        offSpy.mockRestore();
    });
});