(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as anonymous module.
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node / CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    'use strict';

    const NAMESPACE = 'qor.radio';
    const EVENT_ENABLE = 'enable.' + NAMESPACE;
    const EVENT_DISABLE = 'disable.' + NAMESPACE;
    const EVENT_CHANGE = 'change.' + NAMESPACE;

    function QorRadioButton(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorRadioButton.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorRadioButton.prototype = {
        constructor: QorRadioButton,

        init: function() {
            this.bind();
            this.initializeVisibility();
        },

        bind: function() {
            this.$element
                .on(EVENT_CHANGE, 'input[type="radio"]', this.handleChange.bind(this));
        },

        unbind: function() {
            this.$element.off(EVENT_CHANGE);
        },

        initializeVisibility: function() {
            const $checkedRadio = this.$element.find('input[type="radio"]:checked');
            console.log('Initializing visibility for:', this.$element);
            console.log('Checked radio:', $checkedRadio);
            if ($checkedRadio.length) {
                console.log('Handling change for checked radio:', $checkedRadio[0]);
                this.handleChange({ target: $checkedRadio[0] });
            }
        },

        handleChange: function(event) {
            const $radio = $(event.target);
            const $radioGroup = $radio.closest('.radio-group');
            const triggerVisibility = $radioGroup.data('trigger-visibility') === true;
            const isDynamicSetter = $radioGroup.data('is-dynamic-setter') === true;
            const targetClass = $radioGroup.data('target-class');
            const placeholder = $radioGroup.data('placeholder');

            if (triggerVisibility) {
                const name = $radio.attr('name');
                const match = name && name.match(/\[(\d+)\]/);
                
                if (match) {
                    this.onChangeVisibilityCondition(event, match[1]);
                } else {
                    this.onChangeVisibilityCondition(event);
                }
            }
            
            if (isDynamicSetter) {
                this.onChangeDynamicValue(event, targetClass, placeholder, $radioGroup);
            }
        },

        onChangeVisibilityCondition: function(event) {
            const selectedValue = $(event.target).val();
            const $radio = $(event.target);
            
            // Check if we're in a collection edit
            const $collapsibleWrapper = $radio.closest('.collapsible-wrapper');
            let $elementsToToggle;
            
            if ($collapsibleWrapper.length) {
                // If in collection edit, scope to the current wrapper
                $elementsToToggle = $collapsibleWrapper.find('.visibility-by-condition');
            } else {
                // Otherwise, use global scope as before
                $elementsToToggle = $('.visibility-by-condition');
            }
            
            $elementsToToggle.each(function() {
                const $element = $(this);
                const showCondition = $element.data('show-by-condition');
                
                if (showCondition.includes(selectedValue)) {
                    if (!$element.hasClass('hide-on-collapse')) {
                        $element.show();
                    } else {
                        $element.hide();
                    }
                } else {
                    if ($element.hasClass('tykon-arrowup')) {
                        $element.removeClass('tykon-arrowup').addClass('tykon-arrowdown');
                    }
                    $element.hide();
                }
            });
        },

        onChangeDynamicValue: function(event, target, placeholder, $radioGroup) {
            const selectedValue = $(event.target).val();
            const $elementsToUpdate = $(target);

            $elementsToUpdate.each(function() {
                const $element = $(this);
                if ($element.is('input, textarea')) {
                    $element.val($element.val().replace(new RegExp(placeholder, 'g'), selectedValue));
                } else if ($element.is('div')) {
                    $element.text($element.text().replace(new RegExp(placeholder, 'g'), selectedValue));
                } else {
                    $element.text($element.text().replace(new RegExp(placeholder, 'g'), selectedValue));
                }
            });

            $radioGroup.data('placeholder', selectedValue);
        },

        destroy: function() {
            this.unbind();
            this.$element.removeData(NAMESPACE);
        }
    };

    QorRadioButton.DEFAULTS = {};

    QorRadioButton.plugin = function(options) {
        return this.each(function() {
            const $this = $(this);
            let data = $this.data(NAMESPACE);
            let fn;

            if (!data) {
                if (/destroy/.test(options)) {
                    return;
                }
                $this.data(NAMESPACE, (data = new QorRadioButton(this, options)));
            }

            if (typeof options === 'string' && $.isFunction(fn = data[options])) {
                fn.apply(data);
            }
        });
    };

    $(function() {
        const selector = '.radio-group';
        
        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorRadioButton.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorRadioButton.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    $.fn.qorRadioButton = QorRadioButton.plugin;

    return QorRadioButton;
}); 