/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the module loading environment
global.jQuery = $;
global.define = undefined;
global.exports = undefined;
global.module = undefined;

// Now require the module to set up the plugin
require('./qor-radio-button');

describe('QorRadioButton', () => {
    let $element;
    
    beforeEach(() => {
        // Set up our document body
        document.body.innerHTML = `
            <div class="tyk-form-group">
                <label>Test Radio Group</label>
                <div class="radio-group" 
                    data-toggle="qor.radio"
                    data-trigger-visibility="true"
                    data-is-dynamic-setter="true"
                    data-target-class=".target-element"
                    data-placeholder="PLACEHOLDER">
                    <div class="radio-option">
                        <input type="radio" name="test" value="option1">
                        <label class="radio-label" for="test_option1">Option 1</label>
                    </div>
                    <div class="radio-option">
                        <input type="radio" name="test" value="option2">
                        <label class="radio-label" for="test_option2">Option 2</label>
                    </div>
                    <div class="visibility-by-condition" data-show-by-condition="option1" style="display: block;">Show for Option 1</div>
                    <div class="visibility-by-condition" data-show-by-condition="option2" style="display: none;">Show for Option 2</div>
                    <div class="target-element">Text with PLACEHOLDER</div>
                </div>
            </div>
        `;
        
        // Get the element
        $element = $('.radio-group[data-toggle="qor.radio"]');
        
        // Initialize the plugin
        $element.qorRadioButton();
    });

    afterEach(() => {
        $element.qorRadioButton('destroy');
        $element.remove();
        jest.clearAllMocks();
    });

    test('should initialize the radio button', () => {
        expect($element.data('qor.radio')).toBeDefined();
        expect(typeof $element.data('qor.radio')).toBe('object');
    });

    test('should handle visibility control', () => {
        const $option1Radio = $element.find('input[value="option1"]');
        const $option1Content = $element.find('.visibility-by-condition[data-show-by-condition="option1"]');
        const $option2Content = $element.find('.visibility-by-condition[data-show-by-condition="option2"]');

        // Set initial display states
        $option1Content.show();
        $option2Content.hide();

        // Trigger the change event
        $option1Radio.prop('checked', true).trigger('change.qor.radio');

        expect($option1Content.css('display')).not.toBe('none');
        expect($option2Content.css('display')).toBe('none');
    });

    test('should handle dynamic value setting', () => {
        const $dynamicRadio = $element.find('input[value="option1"]');
        const $targetElement = $element.find('.target-element');
        
        // Reset the text content
        $targetElement.text('Text with PLACEHOLDER');
        
        expect($targetElement.text()).toBe('Text with PLACEHOLDER');
        
        // Trigger change event on the radio button
        $dynamicRadio.prop('checked', true).trigger('change.qor.radio');
        
        expect($targetElement.text()).toBe('Text with option1');
    });

    test('should properly destroy the plugin', () => {
        expect($element.data('qor.radio')).toBeDefined();
        
        $element.qorRadioButton('destroy');
        
        expect($element.data('qor.radio')).toBeUndefined();
    });

    test('should handle enable/disable events', () => {
        // First destroy existing instance
        $element.qorRadioButton('destroy');
        
        // Then test enable by triggering document ready and enable event
        $(() => {
            $(document).triggerHandler('enable.qor.radio');
            expect($element.data('qor.radio')).toBeDefined();

            // Then test disable
            $(document).triggerHandler('disable.qor.radio');
            expect($element.data('qor.radio')).toBeUndefined();
        });
    });

    test('should initialize visibility on load with checked radio', () => {
        const $radio = $element.find('input[value="option1"]');
        const $content = $element.find('.visibility-by-condition[data-show-by-condition="option1"]');
        
        // Set initial states
        $radio.prop('checked', true);
        $content.show();
        
        // Reinitialize the plugin
        $element.qorRadioButton('destroy');
        $element.qorRadioButton();
        
        expect($content.css('display')).not.toBe('none');
    });
}); 