(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD. Register as anonymous module.
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // Node / CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals.
        factory(jQuery);
    }
})(function($) {
    'use strict';

    let NAMESPACE = 'qor.rawEditor';
    let EVENT_ENABLE = 'enable.' + NAMESPACE;
    let EVENT_DISABLE = 'disable.' + NAMESPACE;

    function QorRawEditor(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorRawEditor.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorRawEditor.prototype = {
        constructor: QorRawEditor,

        init: function() {
            this.editor = null;
            this.$wizardToggle = $('#wizard-toggle');
            this.$rawEditorToggle = $('#raw-editor-toggle');
            this.$normalSection = $('#normal-section');
            this.$jsonRawEditorWrapper = $('#json-raw-editor-wrapper');
            this.$saveJsonEditorChanges = $('#save-json-editor-changes');
            this.$saveFormChanges = $('#save-form-changes');
            this.$editQorFormContainer = $('.edit-qor-form-container');
            this.initRawEditor();
            this.bind();
        },

        bind: function() {
            $(document).on('click', '#wizard-toggle', this.toggleWizard.bind(this));
            $(document).on('click', '#raw-editor-toggle', this.toggleRawEditor.bind(this));
            $(document).on('click', '#save-json-editor-changes', this.saveChanges.bind(this));
        },

        initRawEditor: function() {
            if (!this.editor && window.ace) {
                this.editor = window.ace.edit("json-raw-editor");
                this.editor.setTheme("ace/theme/chrome");
                this.editor.session.setMode("ace/mode/json");
                this.editor.setOptions({
                    fontSize: "12px",
                    showPrintMargin: false,
                    showGutter: true,
                    highlightActiveLine: false,
                    wrap: false
                });
                this.editor.resize();
            } else if (!window.ace) {
                console.error('Ace editor not loaded');
            }
        },

        updateEditorState: function(mode) {
            if (mode === 'wizard') {
                this.$wizardToggle.addClass('active');
                this.$rawEditorToggle.removeClass('active');
                this.$normalSection.show();
                this.$jsonRawEditorWrapper.hide();
                this.$saveJsonEditorChanges.hide();
                this.$saveFormChanges.show();
                this.$editQorFormContainer.addClass('qor-form-container__border');
            } else if (mode === 'raw') {
                this.$wizardToggle.removeClass('active');
                this.$rawEditorToggle.addClass('active');
                this.$normalSection.hide();
                this.$jsonRawEditorWrapper.show();
                this.$saveFormChanges.hide();
                this.$saveJsonEditorChanges.show();
                this.$editQorFormContainer.removeClass('qor-form-container__border');
            }
        },

        toggleWizard: function(e) {
            e.preventDefault();
            this.updateEditorState('wizard');
        },
    
        toggleRawEditor: function(e) {
            e.preventDefault();
            this.updateEditorState('raw');
            this.initRawEditor();
            this.loadRawEditorContent();
        },

        loadRawEditorContent: function() {
            let url = this.$jsonRawEditorWrapper.data('raw-editor-url');
            fetch(url)
                .then(response => response.json())
                .then(data => {
                    try {
                        if (this.editor) {
                            this.editor.setValue(JSON.stringify(data, null, 2), -1);
                        }
                    } catch (error) {
                        console.error('Failed to parse data as JSON:', error);
                    }
                })
                .catch(error => console.error('Error fetching raw editor content:', error));
        },

        saveChanges: function(e) {
            e.preventDefault();
        
            if (!this.editor) {
                console.error('Editor not initialized');
                return;
            }
            
            let url = this.$jsonRawEditorWrapper.data('raw-editor-url');
            let content = this.editor.getValue();
            let $loading = $(window.QOR.$formLoading);
        
            $.ajax({
                url: url,
                method: 'POST',
                data: content,
                dataType: 'json',
                contentType: 'application/json',
                beforeSend: function() {
                    $(".qor-submit-loading").remove();
                    $loading
                        .appendTo(this.$saveJsonEditorChanges.closest(".qor-form__actions"))
                        .trigger("enable.qor.material");
                }.bind(this),
                success: function(response) {
                    if (response.type === 'success') {
                        localStorage.setItem('flashSuccess', response.message);
                    }
                    let returnUrl = this.$jsonRawEditorWrapper.data('return-url');
                    let refreshUrl = this.$jsonRawEditorWrapper.data('refresh-url');
        
                    if (refreshUrl) {
                        window.location.href = refreshUrl;
                    } else if (returnUrl === "refresh") {
                        window.location.reload();
                    } else if (returnUrl) {
                        window.location.href = returnUrl;
                    } else {
                        window.location.reload();
                    }
                }.bind(this),
                error: function(err) {
                    window.QOR.handleAjaxError(err);
                },
                complete: function() {
                    this.$saveJsonEditorChanges.prop("disabled", false);
                    $loading.remove();
                }.bind(this)
            });
        },
        
        refresh: function() {
            window.location.reload();
        },
        
        load: function(url) {
            window.location.href = url;
        },

        destroy: function() {
            this.unbind();
            this.$element.removeData(NAMESPACE);
        },

        unbind: function() {
            this.$element.off(EVENT_DISABLE);
            $(document).off('click', '#wizard-toggle');
            $(document).off('click', '#raw-editor-toggle');
            $(document).off('click', '#save-json-editor-changes');
        }
    };

    QorRawEditor.DEFAULTS = {};

    QorRawEditor.plugin = function(options) {
        return this.each(function() {
            let $this = $(this);
            let data = $this.data(NAMESPACE);
            let fn;

            if (!data) {
                $this.data(NAMESPACE, (data = new QorRawEditor(this, options)));
            }

            if (typeof options === 'string' && $.isFunction(fn = data[options])) {
                fn.call(data);
            }
        });
    };

    $(function() {
        let selector = '[data-toggle="qor.raw.editor"]';
        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorRawEditor.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorRawEditor.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    $.fn.qorRawEditor = QorRawEditor.plugin;

    return QorRawEditor;
});