// qor-raw-editor.test.js

/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the module loading environment
global.jQuery = $;
global.define = undefined;
global.exports = undefined;
global.module = undefined;

// Now require the module so it uses the global jQuery
require('./qor-raw-editor');

describe('QorRawEditor', () => {
  let $element;
  let instance;
  let editorMock;

  beforeAll(() => {
    // Simulate window.ace
    window.ace = {
      edit: jest.fn(() => {
        editorMock = {
          setTheme: jest.fn(),
          session: {
            setMode: jest.fn(),
          },
          setOptions: jest.fn(),
          resize: jest.fn(),
          setValue: jest.fn(),
          getValue: jest.fn(() => '{}'),
        };
        return editorMock;
      }),
    };

    // Simulate window.QOR
    window.QOR = {
      $formLoading: $('<div class="qor-submit-loading"></div>'),
      handleAjaxError: jest.fn(),
    };

    // Mock console.error
    console.error = jest.fn();
  });

  beforeEach(() => {
    // Set up the DOM elements
    document.body.innerHTML = `
      <div id="wizard-toggle"></div>
      <div id="raw-editor-toggle"></div>
      <div id="normal-section"></div>
      <div id="json-raw-editor-wrapper" data-raw-editor-url="/raw-editor-url" data-return-url="refresh"></div>
      <button id="save-json-editor-changes"></button>
      <button id="save-form-changes"></button>
      <div class="edit-qor-form-container"></div>
      <div id="json-raw-editor"></div>
      <div class="qor-form__actions"></div>
    `;

    $element = $('<div data-toggle="qor.raw.editor"></div>');
    $element.appendTo(document.body);

    // Initialize the plugin
    $element.qorRawEditor();

    // Get the instance
    instance = $element.data('qor.rawEditor');

    // Set initial visibility as per expected initial state
    $('#normal-section').show();
    $('#json-raw-editor-wrapper').hide();
    $('#save-form-changes').show();
    $('#save-json-editor-changes').hide();
  });

  afterEach(() => {
    // Clean up
    $element.qorRawEditor('destroy');
    $element.remove();
    jest.clearAllMocks();
  });

  // Helper function to wait for Promises to resolve
  function flushPromises() {
    return new Promise(resolve => setTimeout(resolve, 0));
  }

  test('should initialize the editor', () => {
    expect(instance).toBeDefined();
    expect(instance.editor).toBeDefined();
  });

  test('should load raw editor content', async () => {
    global.fetch = jest.fn(() =>
      Promise.resolve({
        json: () => Promise.resolve({ key: 'value' }),
      })
    );

    expect(instance.editor).toBe(editorMock);

    instance.loadRawEditorContent();

    // Wait for promises to resolve
    await flushPromises();

    expect(global.fetch).toHaveBeenCalledWith('/raw-editor-url');
    expect(editorMock.setValue).toHaveBeenCalledWith('{\n  "key": "value"\n}', -1);
  });

  test('should handle error when loading raw editor content', async () => {
    global.fetch = jest.fn(() => Promise.reject('Fetch error'));

    instance.loadRawEditorContent();

    // Wait for promises to resolve
    await flushPromises();

    expect(global.fetch).toHaveBeenCalledWith('/raw-editor-url');
    expect(console.error).toHaveBeenCalledWith('Error fetching raw editor content:', 'Fetch error');
  });

  test('should toggle to wizard mode', () => {
    // Set initial state to 'raw' mode
    instance.updateEditorState('raw');

    // Verify initial state is 'raw'
    expect($('#normal-section').css('display')).toBe('none');
    expect($('#json-raw-editor-wrapper').css('display')).not.toBe('none');

    // Click on wizard toggle
    $('#wizard-toggle').trigger('click');

    // Verify that the wizard mode is active
    expect($('#wizard-toggle').hasClass('active')).toBe(true);
    expect($('#raw-editor-toggle').hasClass('active')).toBe(false);
    expect($('#normal-section').css('display')).not.toBe('none');
    expect($('#json-raw-editor-wrapper').css('display')).toBe('none');
    expect($('#save-form-changes').css('display')).not.toBe('none');
    expect($('#save-json-editor-changes').css('display')).toBe('none');
  });

  test('should toggle to raw editor mode', () => {
    // Set initial state to 'wizard' mode
    instance.updateEditorState('wizard');

    // Verify initial state is 'wizard'
    expect($('#normal-section').css('display')).not.toBe('none');
    expect($('#json-raw-editor-wrapper').css('display')).toBe('none');

    // Click on raw editor toggle
    $('#raw-editor-toggle').trigger('click');

    // Verify that the raw editor mode is active
    expect($('#wizard-toggle').hasClass('active')).toBe(false);
    expect($('#raw-editor-toggle').hasClass('active')).toBe(true);
    expect($('#normal-section').css('display')).toBe('none');
    expect($('#json-raw-editor-wrapper').css('display')).not.toBe('none');
    expect($('#save-form-changes').css('display')).toBe('none');
    expect($('#save-json-editor-changes').css('display')).not.toBe('none');
  });

  test('should initialize raw editor when toggled', () => {
    // Spy on initRawEditor and loadRawEditorContent
    instance.initRawEditor = jest.fn();
    instance.loadRawEditorContent = jest.fn();

    // Click on raw editor toggle
    $('#raw-editor-toggle').trigger('click');

    // Verify that initRawEditor and loadRawEditorContent were called
    expect(instance.initRawEditor).toHaveBeenCalled();
    expect(instance.loadRawEditorContent).toHaveBeenCalled();
  });

  test('should save changes successfully', () => {
    editorMock.getValue = jest.fn(() => '{"key":"value"}');

    $.ajax = jest.fn().mockImplementation(({ success, complete }) => {
      success({ type: 'success', message: 'Saved successfully' });
      complete();
    });

    const event = $.Event('click');
    $('#save-json-editor-changes').trigger(event);

    expect(event.isDefaultPrevented()).toBe(true);
    expect($.ajax).toHaveBeenCalledWith(
      expect.objectContaining({
        url: '/raw-editor-url',
        method: 'POST',
        data: '{"key":"value"}',
        dataType: 'json',
        contentType: 'application/json',
      })
    );
  });

  test('should handle error when saving changes', () => {
    editorMock.getValue = jest.fn(() => '{"key":"value"}');

    $.ajax = jest.fn().mockImplementation(({ error, complete }) => {
      error('Error occurred');
      complete();
    });

    const event = $.Event('click');
    $('#save-json-editor-changes').trigger(event);

    expect(window.QOR.handleAjaxError).toHaveBeenCalledWith('Error occurred');
  });

  test('should destroy the plugin', () => {
    instance.unbind = jest.fn();

    instance.destroy();

    expect(instance.unbind).toHaveBeenCalled();
    expect($element.data('qor.rawEditor')).toBeUndefined();
  });
});
