/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock necessary DOM properties
Object.defineProperty(HTMLElement.prototype, 'offsetWidth', {
    get: function() { return 100; }
});

Object.defineProperty(HTMLElement.prototype, 'offsetHeight', {
    get: function() { return 20; }
});

Object.defineProperty(HTMLElement.prototype, 'getClientRects', {
    value: function() { return [{ width: 100, height: 20 }]; }
});

// Mock QOR global object
global.QOR = {
    qorConfirm: jest.fn(),
    $formLoading: '<div class="qor-submit-loading"></div>',
    handleAjaxError: jest.fn()
};

// Mock QOR_Translations
global.QOR_Translations = {
    slideoutCloseWarning: 'Warning: you have unsaved changes!',
    slideoutCheckHTMLTagsError: 'HTML tags are not properly closed'
};

// Mock jQuery plugins if needed
$.fn.modal = jest.fn();
HTMLFormElement.prototype.submit = jest.fn();

const QorSlideout = require('./qor-slideout.js');

describe('QorSlideout', () => {
    let ajaxSpy;
    let element;
    let slideoutInstance;

    beforeEach(() => {
        // Mock jQuery ajax
        ajaxSpy = jest.spyOn($, 'ajax').mockImplementation(() => {
            return {
                done: function() {
                    return this;
                },
                fail: function() {
                    return this;
                }
            };
        });

        // Set up DOM element
        element = $(
            `<div>
                <form>
                    <input type="text" name="required-input" data-required="true" />
                    <input type="text" class="tyk-form-control" data-required="true" />
                    <div class="select2-container" data-required="true">
                        <span class="select2-selection">
                            <span class="select2-selection__rendered">
                                <span class="select2-selection__placeholder">Select an option</span>
                            </span>
                        </span>
                    </div>
                    <div class="radio-group" data-required="true">
                        <input type="radio" name="group1" />
                        <input type="radio" name="group1" />
                    </div>
                </form>
            </div>`
        );

        // Mock window.getComputedStyle
        window.getComputedStyle = jest.fn().mockReturnValue({ display: 'block' });

        // Initialize QorSlideout
        element.qorSlideout({
            url: 'test/url',
            method: 'GET'
        });
        slideoutInstance = element.data('qor.slideout');
    });

    afterEach(() => {
        jest.clearAllMocks();
        ajaxSpy.mockRestore();
    });

    describe('Form Validation', () => {
        test('should validate required input fields and show error message', () => {
            const $form = element.find('form');
            const $input = $form.find('input[name="required-input"]');
            const $tykInput = $form.find('.tyk-form-control');
            
            // Hide other required fields
            window.getComputedStyle = jest.fn(element => {
                if (element === $form.find('.select2-container')[0] || element === $form.find('.radio-group')[0]) {
                    return { display: 'none', visibility: 'hidden' };
                }
                return { display: 'block', visibility: 'visible' };
            });
            
            // Empty inputs
            const emptyResult = slideoutInstance.validateForm($form);
            expect(emptyResult).toBe(false);
            expect($input.hasClass('error-highlight')).toBe(true);
            expect($tykInput.hasClass('error-highlight')).toBe(true);
            
            // Check error message
            const errorMessage = $form.find('.qor-error-message');
            expect(errorMessage.length).toBe(1);
            expect(errorMessage.text()).toBe('Please fill in all required fields');
            
            // Fill regular input
            $input.val('test value');
            expect(slideoutInstance.validateForm($form)).toBe(false);
            expect($input.hasClass('error-highlight')).toBe(false);
            expect($tykInput.hasClass('error-highlight')).toBe(true);
            
            // Fill tyk-form-control input
            $tykInput.val('test value');
            expect(slideoutInstance.validateForm($form)).toBe(true);
            expect($input.hasClass('error-highlight')).toBe(false);
            expect($tykInput.hasClass('error-highlight')).toBe(false);
            
            // Error message should be removed
            expect($form.find('.qor-error-message').length).toBe(0);
        });

        test('should validate select2 dropdowns', () => {
            const $form = element.find('form');
            const $select2Container = $form.find('.select2-container');
            const $select2 = $select2Container.find('.select2-selection__rendered');
            const $placeholder = $select2.find('.select2-selection__placeholder');
            
            // Hide other required fields
            const $input = $form.find('input[name="required-input"]');
            const $tykInput = $form.find('.tyk-form-control');
            const $radioGroup = $form.find('.radio-group');
            window.getComputedStyle = jest.fn(element => {
                if (element === $input[0] || element === $tykInput[0] || element === $radioGroup[0]) {
                    return { display: 'none', visibility: 'hidden' };
                }
                return { display: 'block', visibility: 'visible' };
            });
            
            // Empty select2
            const emptyResult = slideoutInstance.validateForm($form);
            expect(emptyResult).toBe(false);
            expect($select2Container.hasClass('error-highlight')).toBe(true);
            
            // With value
            $placeholder.remove();
            $select2.text('Selected Option');
            const filledResult = slideoutInstance.validateForm($form);
            expect(filledResult).toBe(true);
            expect($select2Container.hasClass('error-highlight')).toBe(false);
        });

        test('should validate radio groups', () => {
            const $form = element.find('form');
            const $radioGroup = $form.find('.radio-group');
            
            // Hide other required fields
            const $input = $form.find('input[name="required-input"]');
            const $tykInput = $form.find('.tyk-form-control');
            const $select2 = $form.find('.select2-container');
            window.getComputedStyle = jest.fn(element => {
                if (element === $input[0] || element === $tykInput[0] || element === $select2[0]) {
                    return { display: 'none', visibility: 'hidden' };
                }
                return { display: 'block', visibility: 'visible' };
            });
            
            // No selection
            const emptyResult = slideoutInstance.validateForm($form);
            expect(emptyResult).toBe(false);
            expect($radioGroup.hasClass('error-highlight')).toBe(true);
            
            // With selection
            $radioGroup.find('input[type="radio"]').first().prop('checked', true);
            const selectedResult = slideoutInstance.validateForm($form);
            expect(selectedResult).toBe(true);
            expect($radioGroup.hasClass('error-highlight')).toBe(false);
        });

        test('should skip validation for hidden elements', () => {
            const $form = element.find('form');
            
            // Mock getComputedStyle to return hidden
            window.getComputedStyle = jest.fn().mockReturnValue({
                display: 'none',
                visibility: 'hidden'
            });
            
            const result = slideoutInstance.validateForm($form);
            expect(result).toBe(true);
        });
    });

    describe('Form Submission', () => {
        test('should prevent submission when validation fails', () => {
            // Set up form element
            const $form = element.find('form');
            $form.attr('action', '/test');
            $form.attr('method', 'POST');
            slideoutInstance.$slideout = element;
            
            // Create submit event
            const submitEvent = $.Event('submit');
            submitEvent.target = $form[0];
            
            // Ensure form validation fails
            window.getComputedStyle = jest.fn().mockReturnValue({ display: 'block' });
            
            // Trigger submit event
            slideoutInstance.submit(submitEvent);
            
            expect(submitEvent.isDefaultPrevented()).toBe(true);
            expect($form.find('.qor-error-message').length).toBe(1);
            expect(ajaxSpy).not.toHaveBeenCalled();
        });

        test('should submit form when validation passes', () => {
            // Set up form element
            const $form = element.find('form');
            $form.attr('action', '/test');
            $form.attr('method', 'post');
            slideoutInstance.$slideout = element;
            
            // Create submit event
            const submitEvent = $.Event('submit');
            submitEvent.target = $form[0];
            
            // Fill all required fields
            $form.find('input[name="required-input"]').val('test value');
            $form.find('.tyk-form-control').val('test value');
            const $select2 = $form.find('.select2-selection__rendered');
            $select2.find('.select2-selection__placeholder').remove();
            $select2.text('Selected Option');
            $form.find('input[type="radio"]').first().prop('checked', true);
            
            // Ensure form validation passes
            window.getComputedStyle = jest.fn().mockReturnValue({ display: 'block' });
            
            // Mock jQuery ajax to capture options
            let capturedOptions;
            ajaxSpy.mockImplementation((url, options) => {
                capturedOptions = options;
                return $.Deferred().promise();
            });
            
            // Trigger submit event
            slideoutInstance.submit(submitEvent);
            
            // Verify ajax was called with correct options
            expect(ajaxSpy).toHaveBeenCalled();
            expect(capturedOptions).toEqual({
                method: 'post',
                url: '/test',
                data: expect.any(FormData),
                dataType: 'html',
                processData: false,
                contentType: false,
                beforeSend: expect.any(Function),
                success: expect.any(Function),
                error: expect.any(Function),
                complete: expect.any(Function)
            });
        });

        test('should handle different return URLs correctly', () => {
            // Set up form element
            const $form = element.find('form');
            $form.attr('action', '/test');
            $form.attr('method', 'post');
            slideoutInstance.$slideout = element;
            
            // Mock window.location
            const originalLocation = window.location;
            delete window.location;
            window.location = { href: '' };
            
            // Mock jQuery ajax to capture success callback
            let successCallback;
            ajaxSpy.mockImplementation((url, options) => {
                successCallback = options.success;
                return {
                    done: function() { return this; },
                    fail: function() { return this; },
                    abort: function() {}
                };
            });
            
            // Mock successful form submission
            const submitEvent = $.Event('submit');
            submitEvent.target = $form[0];
            
            // Fill all required fields to pass validation
            $form.find('input[name="required-input"]').val('test value');
            $form.find('.tyk-form-control').val('test value');
            const $select2 = $form.find('.select2-selection__rendered');
            $select2.find('.select2-selection__placeholder').remove();
            $select2.text('Selected Option');
            $form.find('input[type="radio"]').first().prop('checked', true);
            
            // Test refresh URL
            $form.data('refreshUrl', '/refresh-page');
            slideoutInstance.submit(submitEvent);
            
            // Should redirect to refresh URL
            expect(successCallback).toBeDefined();
            successCallback();
            expect(window.location.href).toBe('/refresh-page');
            
            // Test return URL = "refresh"
            $form.removeData('refreshUrl');
            $form.data('returnUrl', 'refresh');
            const refreshSpy = jest.spyOn(slideoutInstance, 'refresh');
            slideoutInstance.submit(submitEvent);
            successCallback();
            expect(refreshSpy).toHaveBeenCalled();
            
            // Test specific return URL
            $form.data('returnUrl', '/return-page');
            const loadSpy = jest.spyOn(slideoutInstance, 'load');
            slideoutInstance.submit(submitEvent);
            successCallback();
            expect(loadSpy).toHaveBeenCalledWith('/return-page');
            
            // Restore window.location
            window.location = originalLocation;
        });
    });
});