(function (factory) {
  if (typeof define === 'function' && define.amd) {
    // AMD. Register as anonymous module.
    define(['jquery'], factory);
  } else if (typeof exports === 'object') {
    // Node / CommonJS
    factory(require('jquery'));
  } else {
    // Browser globals.
    factory(jQuery);
  }
})(function ($) {

  'use strict';

  let NAMESPACE = 'qor.tabs';
  let EVENT_ENABLE = 'enable.' + NAMESPACE;
  let EVENT_DISABLE = 'disable.' + NAMESPACE;
  let EVENT_CLICK = 'click.' + NAMESPACE;

  function QorTabs(element, options) {
    this.$element = $(element);
    this.options = $.extend({}, QorTabs.DEFAULTS, $.isPlainObject(options) && options);
    
    // Store the form content template during initialization
    const $template = this.$element.find('.qor-fieldset--new');
    if ($template.length) {
        // Store template specifically from the dynamic collection
        const $dynamicTemplate = $template.closest('.dynamic-collection').find('.qor-fieldset--new');
        if ($dynamicTemplate.length) {
            this.$formContentTemplate = $dynamicTemplate.find('.qor-form-section');
        }
    }
    
    this.openModalWithImage = function(event, imageName) {
        const existingModal = document.getElementById('imageModal');
        if (existingModal) existingModal.remove();

        const modal = document.createElement('div');
        modal.id = 'imageModal';
        modal.className = 'modal';
        modal.style.display = 'block';

        const modalContent = document.createElement('div');
        modalContent.className = 'modal-content';

        const closeButton = document.createElement('img');
        closeButton.className = 'close';
        closeButton.src = '/admin/assets/images/xmark.svg';
        closeButton.onclick = this.closeModal;

        const modalImage = document.createElement('img');
        modalImage.id = 'modalImage';
        modalImage.src = `/admin/assets/images/${imageName}`;
        modalImage.alt = 'Modal Image';

        modalContent.appendChild(closeButton);
        modalContent.appendChild(modalImage);
        modal.appendChild(modalContent);
        document.body.appendChild(modal);
    };

    this.closeModal = function() {
        const modal = document.getElementById('imageModal');
        if (modal) {
            modal.remove();
        }
    };

    // Use mousedown instead of click and prevent default immediately
    $(document).on('mousedown.modalImage', '[data-modal-trigger]', (e) => {
        e.preventDefault();
        e.stopPropagation();
        e.stopImmediatePropagation();
        return false;
    });

    // Handle the actual modal opening on mouseup
    $(document).on('mouseup.modalImage', '[data-modal-trigger]', (e) => {
        const button = $(e.currentTarget);
        const imageName = button.data('image');
        this.openModalWithImage(e, imageName);
        e.preventDefault();
        e.stopPropagation();
        e.stopImmediatePropagation();
        return false;
    });

    // Prevent any click events just in case
    $(document).on('click.modalImage', '[data-modal-trigger]', (e) => {
        e.preventDefault();
        e.stopPropagation();
        e.stopImmediatePropagation();
        return false;
    });

    this.init();
  }

  QorTabs.prototype = {
    constructor: QorTabs,

    init: function () {
      this.$tabButtons = this.$element.find('.qor-tabs__tab');
      this.$tabPanels = this.$element.find('.qor-tabs__panel');
      
      this.bind();
      this.initializeDynamicPanels();
      this.setupRevealLogic();
    },

    bind: function () {
      this.$element.on(EVENT_CLICK, '.qor-tabs__tab', this.switchTab.bind(this));
    },

    switchTab: function (e) {
      e.preventDefault();
      let $target = $(e.currentTarget);
      let tabIndex = $target.data('tab');

      // Remove active class from all buttons and panels
      this.$tabButtons.removeClass('is-active');
      this.$tabPanels.removeClass('is-visible');

      // Add active class to clicked button and corresponding panel
      $target.addClass('is-active');
      const $panel = this.$element.find(`.qor-tabs__panel[data-panel="${tabIndex}"]`);
      $panel.addClass('is-visible');

      // Handle dynamic content if panel is dynamic
      if ($panel.data('dynamic')) {
        this.handleDynamicPanel($panel);
        this.$element.trigger('contentLoaded.qor.tab', [$panel, true]);
      }
    },

    setupRevealLogic: function () {
      const self = this;
      this.$tabPanels.each(function () {
        const $panel = $(this);
        const revealTarget = $panel.data('reveal-target');
        
        if (revealTarget) {
          const $select = $(`select[name="${revealTarget}"]`);
          
          if ($select.length) {
            // Remove any existing change handlers to prevent duplicates
            $select.off('change.qor.tabs.reveal')
                   .on('change.qor.tabs.reveal', function () {
              self.updatePanelVisibility($panel, $select);
            });
            
            // Initial visibility check
            self.updatePanelVisibility($panel, $select, true);
          }
        }
      });
    },

    updatePanelVisibility: function ($panel, $select) {
      const conditions = $select.data('reveal-conditions') || [];
      const matchingCondition = conditions.find(condition => condition.Value === $select.val());
      
      // Only manage tab button visibility, not panel content
      const tabIndex = $panel.data('panel');
      const $tabButton = this.$tabButtons.filter(`[data-tab="${tabIndex}"]`);
      
      if (matchingCondition) {
        $tabButton.show();
      } else {
        $tabButton.hide();
        // If the panel was visible and we're hiding its tab, hide the panel too
        if ($panel.hasClass('is-visible')) {
          $panel.removeClass('is-visible');
        }
      }
    },

    initializeDynamicPanels: function() {
      // Initialize visible dynamic panels on page load
      const $visiblePanel = this.$tabPanels.filter('.is-visible[data-dynamic="true"]');
      if ($visiblePanel.length) {
        this.handleDynamicPanel($visiblePanel);
      }
    },

    handleDynamicPanel: function($panel) {
      const dependsOnSelector = $panel.data('depends-on');
      const $dependentElement = $(dependsOnSelector);
      
      if ($dependentElement.length) {
        this.updateDynamicContent($panel, $dependentElement);
      }
    },

    updateDynamicContent: function($panel, $dependentElement) {
      const $collection = $panel.find('.dynamic-collection');
      const $dynamicContent = $collection.find('.dynamic-content');
      const $addButton = $collection.find('.qor-fieldset__add');
      const $template = $collection.find('.qor-fieldset--new');

      const selectedItems = [];
      $dependentElement.find('.select2-selection__choice').each(function() {
        selectedItems.push({
          title: $(this).attr('title'),
          id: $(this).attr('item-id')
        });
      });
      
      if (selectedItems.length) {
        // Remove manually added fieldsets (siblings with empty data-value)
        $dynamicContent.siblings('.qor-fieldset').filter(function() {
          return !$(this).data('value') && !$(this).hasClass('qor-fieldset--new');
        }).remove();
        
        // Remove existing fieldsets that don't match selected items
        $dynamicContent.siblings('.qor-fieldset').filter(function() {
          const fieldsetValue = $(this).data('value');
          return fieldsetValue && !selectedItems.some(item => item.id === fieldsetValue);
        }).remove();
        
        // Remove dynamically added fieldsets inside dynamic-content that don't match selected items
        $dynamicContent.find('.qor-fieldset').filter(function() {
          const fieldsetValue = $(this).data('value');
          return fieldsetValue && !selectedItems.some(item => item.id === fieldsetValue);
        }).remove();
        
        $addButton.hide();
        this.renderDynamicContent($panel, $dynamicContent, $template, selectedItems);
      } else {
        // Remove all manually added fieldsets and dynamic content fieldsets
        $dynamicContent.siblings('.qor-fieldset').filter(function() {
          return $(this).data('value') && !$(this).hasClass('qor-fieldset--new');
        }).remove();
        $dynamicContent.find('.qor-fieldset').remove();
        $addButton.show();
      }
    },

    renderDynamicContent: function($panel, $dynamicContent, $template, selectedItems) {
      const setsValueOf = $panel.data('sets-value');
      
      // Get existing fieldsets and their values (including siblings)
      const existingFieldsets = $dynamicContent.siblings('.qor-fieldset').add($dynamicContent.find('.qor-fieldset')).not('.qor-fieldset--new');
      const existingValues = new Map();
      const existingItems = new Set();

      existingFieldsets.each(function() {
        const $fieldset = $(this);
        const value = $fieldset.data('value');
        if (value) {
          existingItems.add(value);
          const inputName = $fieldset.find('input').first().attr('name');
          const match = inputName ? inputName.match(/\[(\d+)\]/) : null;
          const index = match ? parseInt(match[1]) : null;
          existingValues.set(value, {
            fieldset: $fieldset,
            index: index
          });
        }
      });

      let maxIndex = 1;
      existingValues.forEach(({index}) => {
        if (index && index > maxIndex) {
          maxIndex = index + 1;
        }
      });

      // Only create dynamic fieldsets for items that don't have existing fieldsets
      selectedItems.forEach(item => {
        if (!existingItems.has(item.id)) {
          maxIndex++;
          
          const $newFieldset = $('<fieldset>').addClass('qor-fieldset')
            .attr('data-value', item.id);

          const $deleteBtn = $('<button>')
            .attr({
              'data-confirm': 'Are you sure?',
              'type': 'button'
            })
            .addClass('mdl-button qor-button--muted mdl-button--icon mdl-js-button qor-fieldset__delete tyk-trash-icon');
          
          $newFieldset.append($deleteBtn);

          const $collapsibleWrapper = $('<div>').addClass('collapsible-wrapper');
          
          const $header = $('<h3>')
            .addClass('collapse-cta')
            .attr('onclick', 'toggleCollapsible(this)')
            .html(`
              <i class="tyk-icon tykon tykon-arrowdown collapsible-icon"></i>
              <span class="dynamic-label">${item.title}</span>
            `);

          const $content = $('<div>')
            .addClass('collapse-content')
            .css('display', 'none');

          // Use stored template with deep clone to preserve events and data
          const $formContent = this.$formContentTemplate.clone(true, true);
          $formContent.find('input, select, textarea, label').each(function() {
            let $elem = $(this);
            let name = $elem.attr('name');
            if (name) {
              $elem.attr('name', name.replace('[0]', `[${maxIndex}]`));
            }
            let id = $elem.attr('id');
            if (id) {
              $elem.attr('id', id.replace('_0_', `_${maxIndex}_`));
            }
            let forAttr = $elem.attr('for');
            if (forAttr) {
              $elem.attr('for', forAttr.replace('_0_', `_${maxIndex}_`));
            }
          });

          // Set reference ID and Name if needed
          if (setsValueOf) {
            const selectors = setsValueOf.split(',');
            
            // Set ReferenceID
            const $referenceInput = $formContent.find(selectors[0]);
            if ($referenceInput.length) {
              $referenceInput.val(item.id);
            }

            // Set Name
            if (selectors[1]) {
              const $nameInput = $formContent.find(selectors[1]);
              if ($nameInput.length) {
                $nameInput.val(item.title);
              }
            }
          }

          $content.append($formContent);
          $collapsibleWrapper.append($header).append($content);
          $newFieldset.append($collapsibleWrapper);

          $dynamicContent.append($newFieldset);

          // Initialize QorRadioButton for the new fieldset
          $newFieldset.find('.radio-group').qorRadioButton();
          
          // Trigger cropper initialization for file inputs
          $newFieldset.trigger('enable.qor.cropper');
        }
      });

      if (this.options.maintainOrder) {
        this.reorderFieldsets($dynamicContent, selectedItems);
      }
    },

    unbind: function () {
      this.$element.off(EVENT_CLICK, '.qor-tabs__tab');
    },

    destroy: function () {
      $(document).off('.modalImage');
      this.closeModal();
      this.unbind();
      this.$element.removeData(NAMESPACE);
    }
  };

  QorTabs.DEFAULTS = {};

  QorTabs.plugin = function (options) {
    return this.each(function () {
      let $this = $(this);
      let data = $this.data(NAMESPACE);
      let fn;

      if (!data) {
        if (/destroy/.test(options)) {
          return;
        }
        data = new QorTabs(this, options);
        $this.data(NAMESPACE, data);
      }

      if (typeof options === 'string' && $.isFunction(fn = data[options])) {
        fn.call(data);
      }
    });
  };

  $(function () {
    let selector = '[data-toggle="qor.tabs"]';
    $(document)
      .on(EVENT_DISABLE, function (e) {
        QorTabs.plugin.call($(selector, e.target), 'destroy');
      })
      .on(EVENT_ENABLE, function (e) {
        QorTabs.plugin.call($(selector, e.target));
      })
      .triggerHandler(EVENT_ENABLE);
  });

  $.fn.qorTabs = QorTabs.plugin;

  return QorTabs;
});
