/**
 * @jest-environment jsdom
 */

const $ = require('jquery');

// Mock the module loading environment
global.jQuery = $;
global.define = undefined;
global.exports = undefined;
global.module = undefined;

// Mock qorRadioButton
$.fn.qorRadioButton = function() {
    return this;
};

// Now require the module to set up the plugin and capture the QorTabs class
const QorTabs = require('./qor-tabs');

describe('QorTabs', () => {
    let $element;
    let qorTabs;
    
    beforeEach(() => {
        // Set up the HTML structure
        document.body.innerHTML = `
            <div class="qor-tabs__container" data-toggle="qor.tabs">
                <div class="qor-tabs__header">
                    <button type="button" class="qor-tabs__tab is-active" data-tab="0" data-target="#tab-content-0">Details</button>
                    <button type="button" class="qor-tabs__tab" data-tab="1" data-target="#tab-content-1">API's</button>
                    <button type="button" class="qor-tabs__tab" data-tab="2" data-target="#tab-content-2">Documentation</button>
                    <button type="button" class="qor-tabs__tab" data-tab="3" data-target="#tab-content-3">Getting started</button>
                    <button type="button" class="qor-tabs__tab" data-tab="4" data-target="#tab-content-4" data-reveal-target="dcr-section">DCR</button>
                </div>
                <div class="qor-tabs__content">
                    <div id="tab-content-0" class="qor-tabs__panel is-visible" data-panel="0">
                        <div class="qor-form-section">
                            <div class="select2-selection__choice" title="Option 1" item-id="1"></div>
                            <div class="qor-fieldset">Old content</div>
                            <input type="text" name="field1" value="value1">
                        </div>
                    </div>
                    <div id="tab-content-1" class="qor-tabs__panel" data-panel="1" data-dynamic="true" data-depends-on="#tab-content-0">
                        <div class="dynamic-collection">
                            <div class="dynamic-content"></div>
                            <div class="qor-fieldset--new">
                                <div class="qor-form-section">
                                    <input type="text" name="test[0][field]">
                                </div>
                            </div>
                        </div>
                    </div>
                    <div id="tab-content-2" class="qor-tabs__panel" data-panel="2"></div>
                    <div id="tab-content-3" class="qor-tabs__panel" data-panel="3"></div>
                    <div id="tab-content-4" class="qor-tabs__panel" data-panel="4"></div>
                </div>
                <div class="dcr-section" style="display: none;">DCR Content</div>
            </div>
        `;
        
        $element = $('.qor-tabs__container');
        
        // Import and initialize the actual QorTabs plugin
        require('./qor-tabs.js');
        $element.qorTabs();
    });

    test('should switch tabs on click', () => {
        const $tabs = $element.find('.qor-tabs__tab');
        const $panels = $element.find('.qor-tabs__panel');

        $tabs.eq(1).trigger('click');
        expect($tabs.eq(1).hasClass('is-active')).toBeTruthy();
        expect($panels.eq(1).hasClass('is-visible')).toBeTruthy();
        expect($panels.eq(0).hasClass('is-visible')).toBeFalsy();
    });

    test('should initialize dynamic panels', () => {
        // Set up a dynamic panel that should be visible initially
        const $dynamicPanel = $element.find('#tab-content-1');
        $dynamicPanel.attr('data-dynamic', 'true');
        $dynamicPanel.addClass('is-visible');
        
        // Re-initialize QorTabs using the jQuery plugin
        $element.qorTabs('destroy').qorTabs();
        
        // Get the QorTabs instance
        const qorTabs = $element.data('qor.tabs');
        
        // Verify the panel remains visible and has dynamic content
        expect($dynamicPanel.hasClass('is-visible')).toBeTruthy();
        expect($dynamicPanel.data('dynamic')).toBeTruthy();
    });

    test('should handle dynamic panel loading', () => {
        const $dynamicTab = $element.find('.qor-tabs__tab').eq(1);
        const $targetPanel = $element.find('.qor-tabs__panel').eq(1);
        
        // Set up the panel with required data attributes
        $targetPanel.attr({
            'data-dynamic': 'true',
            'data-depends-on': '#tab-content-0'
        });

        // Add content to dependent element
        const $dependentElement = $element.find('#tab-content-0');
        $dependentElement.find('.select2-selection__choice').attr({
            'title': 'Test Item',
            'item-id': '123'
        });

        // Trigger tab switch
        $dynamicTab.trigger('click');

        // Verify dynamic content was added
        expect($targetPanel.find('.dynamic-content .qor-fieldset').length).toBeGreaterThan(0);
    });

    test('should update dynamic content', () => {
        const $panel = $element.find('#tab-content-0 .qor-form-section');
        const $dependentElement = $element.find('.select2-selection__choice');
        
        const qorTabs = $element.data('qor.tabs');
        qorTabs.updateDynamicContent($panel, $dependentElement);
        
        // Verify content was updated
        expect($panel.html()).toContain('Option 1');
    });

    test('should render dynamic content with existing values', () => {
        const $content = $element.find('#tab-content-0 .qor-form-section');
        const $template = $element.find('.qor-fieldset--new');
        const selectedItems = [{ title: 'Option 1', id: '1' }];
        
        const qorTabs = $element.data('qor.tabs');
        qorTabs.renderDynamicContent($content, $content, $template, selectedItems);
        
        // Verify content was rendered and values preserved
        expect($content.find('input[name="field1"]').val()).toBe('value1');
    });

    test('should handle error states in dynamic loading', () => {
        const $panel = $element.find('#tab-content-3');
        
        // Set up panel with dynamic content structure
        $panel.html(`
            <div class="dynamic-collection">
                <div class="dynamic-content"></div>
                <div class="qor-fieldset--new">
                    <div class="qor-form-section">
                        <input type="text" name="test[0][field]">
                    </div>
                </div>
            </div>
        `);
        
        // Set up panel with invalid dependency
        $panel.attr({
            'data-dynamic': 'true',
            'data-depends-on': '#non-existent-element'
        });
        
        const qorTabs = $element.data('qor.tabs');
        qorTabs.handleDynamicPanel($panel);
        
        // Since there's no dependent element, the dynamic content should be empty
        expect($panel.find('.dynamic-content .qor-fieldset').length).toBe(0);
        expect($panel.find('.qor-fieldset__add').css('display')).not.toBe('none');
    });

    test('should destroy properly', () => {
        const qorTabs = $element.data('qor.tabs');
        
        // Verify event handlers are bound
        expect($._data($element[0], 'events')).toBeDefined();
        
        // Call destroy
        qorTabs.destroy();
        
        // Verify event handlers are removed
        expect($._data($element[0], 'events')).toBeUndefined();
        expect($element.data('qor.tabs')).toBeUndefined();
    });

    // Modal tests
    describe('Modal Functionality', () => {
        beforeEach(() => {
            // Set up specific HTML for modal tests
            document.body.innerHTML = `
                <div class="qor-tabs__container" data-toggle="qor.tabs">
                    <div class="qor-tabs__header">
                        <button type="button" class="qor-tabs__tab" data-tab="1" data-target="#tab-content-1">
                            API's
                        </button>
                    </div>
                    <div class="qor-tabs__content">
                        <div id="tab-content-1" class="qor-tabs__panel" data-panel="1">
                            <div class="qor-form-section clearfix">
                                <button type="button" class="tyk-button tyk-button--primary-outline tyk-button--md live-portal-example" data-modal-trigger="" data-image="live-example.png">Live portal example</button>
                            </div>
                        </div>
                    </div>
                </div>
            `;

            // Initialize QorTabs using the jQuery plugin method
            $element = $('.qor-tabs__container').qorTabs(); // Initialize the plugin
        });

        test('should open modal with image when button is clicked', () => {
            // Simulate a mouseup event on the button that should open the modal
            const $button = $element.find('.live-portal-example');
            $button.trigger('mouseup'); // Trigger mouseup event

            // Check if the modal is created
            const modal = document.getElementById('imageModal');
            expect(modal).toBeTruthy(); // Check if modal exists
            expect(modal.style.display).toBe('block'); // Check if modal is visible

            // Check if the image is set correctly
            const modalImage = document.getElementById('modalImage');
            expect(modalImage).toBeTruthy(); // Check if modal image exists
            expect(modalImage.src).toContain('live-example.png'); // Check if the image source is correct

            // Check if the close button is present
            const closeButton = modal.querySelector('.close');
            expect(closeButton).toBeTruthy(); // Check if close button exists
        });

        test('should close modal when close button is clicked', () => {
            // Open the modal first
            const $button = $element.find('.live-portal-example');
            $button.trigger('mouseup'); // Trigger mouseup event

            // Simulate a click on the close button
            const closeButton = document.querySelector('.close');
            closeButton.click();

            // Check if the modal is removed
            const modal = document.getElementById('imageModal');
            expect(modal).toBeNull(); // Check if modal is removed
        });

        test('should prevent default on mousedown event', () => {
            const $button = $element.find('.live-portal-example');
            const event = $.Event('mousedown', { currentTarget: $button[0] });
            $button.trigger(event);

            // Check that the default action is prevented
            expect(event.isDefaultPrevented()).toBe(true);
        });

        test('should prevent default on click event', () => {
            const $button = $element.find('.live-portal-example');
            const event = $.Event('click', { currentTarget: $button[0] });
            $button.trigger(event);

            // Check that the default action is prevented
            expect(event.isDefaultPrevented()).toBe(true);
        });
    });

    test('should handle existing fieldsets and calculate maxIndex correctly', () => {
        const $content = $element.find('#tab-content-0 .qor-form-section');
        const $template = $element.find('.qor-fieldset--new');
        const selectedItems = [{ title: 'Option 1', id: '1' }, { title: 'Option 2', id: '2' }];

        // Add existing fieldsets with data-value and input names
        $content.append(`
            <fieldset class="qor-fieldset" data-value="1">
                <input type="text" name="test[1][field]">
            </fieldset>
        `);

        const qorTabs = $element.data('qor.tabs');
        qorTabs.renderDynamicContent($content, $content, $template, selectedItems);

        // Verify that maxIndex is calculated correctly
        const newFieldset = $content.find('.qor-fieldset[data-value="2"]');
        expect(newFieldset.length).toBe(1);
        expect(newFieldset.find('input').attr('name')).toContain('[3]');
    });

    test('should set reference ID and Name if needed', () => {
        // Set up the initial HTML structure
        document.body.innerHTML = `
            <div class="qor-tabs__container" data-toggle="qor.tabs">
                <div class="qor-tabs__header">
                    <button type="button" class="qor-tabs__tab" data-tab="0" data-target="#tab-content-0">Tab</button>
                </div>
                <div class="qor-tabs__content">
                    <div id="tab-content-0" class="qor-tabs__panel" data-panel="0" data-sets-value="input[name*='reference_id'],input[name*='name']">
                        <div class="qor-form-section">
                            <div class="dynamic-collection">
                                <div class="dynamic-content"></div>
                                <div class="qor-fieldset--new">
                                    <div class="qor-form-section">
                                        <input type="text" name="test[0][reference_id]">
                                        <input type="text" name="test[0][name]">
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        `;

        const $element = $('.qor-tabs__container');
        
        // Initialize QorTabs
        $element.qorTabs();
        const qorTabs = $element.data('qor.tabs');
        
        // Get references to the elements we need
        const $panel = $('#tab-content-0');
        const $dynamicContent = $panel.find('.dynamic-content');
        const $template = $panel.find('.qor-fieldset--new');
        
        console.log('Initial template HTML:', $template.html());
        console.log('Form content template:', qorTabs.$formContentTemplate.html());
        
        // Call renderDynamicContent with a selected item
        const selectedItems = [{ title: 'Option 1', id: '1' }];
        qorTabs.renderDynamicContent(
            $panel,
            $dynamicContent,
            $template,
            selectedItems
        );
        
        // Find the newly created fieldset and verify its inputs
        const $newFieldset = $dynamicContent.find('.qor-fieldset[data-value="1"]');
        console.log('Created fieldset HTML:', $newFieldset.html());
        
        const $referenceInput = $newFieldset.find('input[name*="reference_id"]');
        const $nameInput = $newFieldset.find('input[name*="name"]');
        
        expect($referenceInput.length).toBe(1, 'Reference input should exist');
        expect($nameInput.length).toBe(1, 'Name input should exist');
        expect($referenceInput.val()).toBe('1', 'Reference ID should be set to 1');
        expect($nameInput.val()).toBe('Option 1', 'Name should be set to Option 1');
    });

    afterEach(() => {
        // Clean up the DOM after each test
        document.body.innerHTML = '';
    });
});

describe('Reveal Logic in QorTabs', () => {
    let $element;
    
    beforeEach(() => {
        // Set up the HTML structure
        document.body.innerHTML = `
            <div class="qor-tabs__container" data-toggle="qor.tabs">
                <div class="qor-tabs__header">
                    <button type="button" class="qor-tabs__tab" data-tab="0" data-target="#tab-content-0">Details</button>
                    <button type="button" class="qor-tabs__tab" data-tab="1" data-target="#tab-content-1" data-reveal-target="authMethod" style="display: none;">API's</button>
                </div>
                <div class="qor-tabs__content">
                    <div id="tab-content-0" class="qor-tabs__panel" data-panel="0"></div>
                    <div id="tab-content-1" class="qor-tabs__panel" data-panel="1" data-reveal-target="authMethod"></div>
                </div>
                <select name="authMethod" data-reveal-conditions='[{"Value": "api", "Target": "authMethod"}]'>
                    <option value="none">None</option>
                    <option value="api">API</option>
                </select>
            </div>
        `;
        
        $element = $('.qor-tabs__container');
        
        // Import and initialize the actual QorTabs plugin
        require('./qor-tabs.js');
        $element.qorTabs();
    });

    test('should show/hide tab button based on select value', () => {
        const $select = $('select[name="authMethod"]');
        const $panel = $('#tab-content-1');
        const $tabButton = $element.find('.qor-tabs__tab[data-tab="1"]');

        // Initially, the tab button should be hidden
        expect($tabButton.css('display')).toBe('none');
        expect($panel.hasClass('is-visible')).toBeFalsy();

        // Change the select value to match the reveal condition
        $select.val('api').trigger('change');

        // The tab button should now be visible, but panel should still be hidden
        expect($tabButton.css('display')).not.toBe('none');
        expect($panel.hasClass('is-visible')).toBeFalsy();

        // Click the tab button
        $tabButton.trigger('click');

        // Now the panel should be visible
        expect($panel.hasClass('is-visible')).toBeTruthy();
    });

    test('should hide tab button and panel when select value changes to non-matching condition', () => {
        const $select = $('select[name="authMethod"]');
        const $panel = $('#tab-content-1');
        const $tabButton = $element.find('.qor-tabs__tab[data-tab="1"]');

        // Set up initial state with matching condition and visible panel
        $select.val('api').trigger('change');
        expect($tabButton.css('display')).not.toBe('none');
        
        $tabButton.trigger('click');
        expect($panel.hasClass('is-visible')).toBeTruthy();

        // Change select to non-matching value
        $select.val('none').trigger('change');

        // Both tab button and panel should be hidden
        expect($tabButton.css('display')).toBe('none');
        expect($panel.hasClass('is-visible')).toBeFalsy();
    });

    afterEach(() => {
        // Clean up the DOM after each test
        document.body.innerHTML = '';
    });
}); 