(function(factory) {
    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        module.exports = factory(require('jquery'));
    } else {
        factory(jQuery);
    }
})(function($) {
    'use strict';

    let NAMESPACE = 'qor.wizard';
    let EVENT_ENABLE = 'enable.' + NAMESPACE;
    let EVENT_DISABLE = 'disable.' + NAMESPACE;

    function QorWizard(element, options) {
        this.$element = $(element);
        this.options = $.extend({}, QorWizard.DEFAULTS, $.isPlainObject(options) && options);
        this.init();
    }

    QorWizard.prototype = {
        constructor: QorWizard,

        init: function() {
            this.bind();
            this.initWizard();
        },

        bind: function() {
            this.$element.on(EVENT_DISABLE, this.destroy.bind(this));
        },

        initWizard: function() {
            let $element = this.$element[0];
            let steps = $element.querySelectorAll('.qor-wizard__step');
            let prevButton = $element.querySelector('.qor-wizard__prev');
            let nextButton = $element.querySelector('.qor-wizard__next');
            let currentStep = 0;

            function updateStepSummary(step) {
                let summaryElements = step.querySelectorAll('.show-in-summary');
                let inlineSummary = [];
                let multilineSummary = [];
            
                let prevStep = step.previousElementSibling;
                let showByCondition = null;
                if (prevStep) {
                    let triggerElement = prevStep.querySelector('[data-trigger-visibility="true"]');
                    if (triggerElement && triggerElement.tagName === 'DIV') {
                        let selectedRadio = triggerElement.querySelector('input[type="radio"]:checked');
                        if (selectedRadio) {
                            showByCondition = selectedRadio.value;
                        }
                    }
                }
            
                summaryElements.forEach(function(element) {
                    if (showByCondition && element.dataset.showByCondition) {
                        if (!element.dataset.showByCondition.includes(showByCondition)) {
                            return;
                        }
                    }
                    let labelClass = element.dataset.summaryLabelClass;
                    let valueClass = element.dataset.summaryValueClass;
                    let labelElement = labelClass ? element.querySelector(labelClass) : null;
                    let valueElement = valueClass ? element.querySelector(valueClass) : null;
            
                    let label = labelElement ? labelElement.textContent.trim() : '';
                    let value = '';
            
                    if (valueElement) {
                        if (valueElement.tagName === 'INPUT') {
                            value = valueElement.value;
                        } else if (valueElement.classList.contains('radio-group')) {
                            let selectedRadio = valueElement.querySelector('input[type="radio"]:checked');
                            if (selectedRadio) {
                                let radioLabel = selectedRadio.nextElementSibling;
                                let firstParagraph = radioLabel.querySelector('span');
                                value = firstParagraph ? firstParagraph.textContent.trim() : '';
                            }
                        } else {
                            value = valueElement.textContent.trim();
                        }
                    }
            
                    if (value) {
                        let summaryText = label ? label.replace(/\*/g, '').trim() + ' | ' + value : value;
                        
                        if (element.classList.contains('summary-inline')) {
                            inlineSummary.push(summaryText);
                        } else {
                            multilineSummary.push(summaryText);
                        }
                    }
                });
            
                let summaryContainer = step.querySelector('.qor-wizard__step-summary');
                summaryContainer.innerHTML = '';
            
                if (inlineSummary.length > 0) {
                    let inlineContainer = document.createElement('div');
                    inlineContainer.className = 'qor-wizard__inline-summary';
                    inlineContainer.style.display = 'flex';
                    inlineContainer.style.flexWrap = 'wrap';
                    inlineContainer.style.gap = '60px';
                    inlineSummary.forEach(function(summary) {
                        let summaryDiv = document.createElement('div');
                        summaryDiv.className = 'qor-wizard__summary-item';
                        summaryDiv.textContent = summary;
                        inlineContainer.appendChild(summaryDiv);
                    });
            
                    summaryContainer.appendChild(inlineContainer);
                }
            
                if (multilineSummary.length > 0) {
                    if (inlineSummary.length > 0) {
                        summaryContainer.appendChild(document.createElement('br'));
                    }
                    multilineSummary.forEach(function(summary) {
                        summaryContainer.appendChild(document.createTextNode(summary));
                        summaryContainer.appendChild(document.createElement('br'));
                    });
                }
            }
            function showStep(stepIndex) {
                steps.forEach(function(step, index) {
                    if (index <= stepIndex) {
                        step.classList.add('is-visible');
                    } else {
                        step.classList.remove('is-visible');
                    }
                    
                    if (index === stepIndex) {
                        step.classList.add('is-active');
                        step.classList.remove('is-completed');
                        step.querySelector('.qor-wizard__step-content').style.display = 'block';
                    } else if (index < stepIndex) {
                        step.classList.add('is-completed');
                        step.classList.remove('is-active');
                        step.querySelector('.qor-wizard__step-content').style.display = 'none';
                        updateStepSummary(step);
                        step.querySelector('.qor-wizard__step-number').innerHTML = '✓';
                    } else {
                        step.classList.remove('is-active', 'is-completed');
                        step.querySelector('.qor-wizard__step-content').style.display = 'none';
                        step.querySelector('.qor-wizard__step-number').innerHTML = index + 1;
                    }
                });
                if ((stepIndex > 0 || currentStep > 0) && !prevButton.classList.contains('is-visible')) {
                    prevButton.classList.add('is-visible');
                }
            }

            function isElementVisible(element) {
                return !!(element.offsetWidth || element.offsetHeight || element.getClientRects().length);
            }

            function isStepValid(step) {
                let requiredInputs = step.querySelectorAll('input[data-required="true"], div[data-required="true"] .select2-selection__rendered');
                let valid = true;
                for (let i = 0; i < requiredInputs.length; i++) {
                    if (isElementVisible(requiredInputs[i])) {
                        if (requiredInputs[i].tagName === 'INPUT') {
                            if (!requiredInputs[i].value.trim()) {
                                requiredInputs[i].classList.add('error-highlight');
                                valid = false;  
                            }
                        } else {
                            let placeholder = requiredInputs[i].querySelector('.select2-selection__placeholder');
                            if (placeholder || !requiredInputs[i].innerText.trim()) {
                                requiredInputs[i].parentElement.classList.add('error-highlight');
                                valid = false;
                            }
                        }
                    }
                }

                let radioGroups = step.querySelectorAll('div[data-required="true"].radio-group');
                for (let i = 0; i < radioGroups.length; i++) {
                    if (isElementVisible(radioGroups[i])) {
                        let checkedRadio = radioGroups[i].querySelector('input[type="radio"]:checked');
                        if (!checkedRadio) return false;
                    }
                }

                return valid;
            }

            function nextStep() {
                if (currentStep < steps.length - 1) {
                    let currentStepElement = steps[currentStep];
                    if (isStepValid(currentStepElement)) {
                        currentStep++;
                        showStep(currentStep);
                    } else {
                        return;
                    }
                } else {
                    if (isStepValid(steps[currentStep])) {
                        let $form = $($element.closest('form'));
                        $form.trigger("submit.qor.slideout");
                    } else {
                        return;
                    }
                }
            }

            function prevStep() {
                if (currentStep > 0) {
                    currentStep--;
                    showStep(currentStep);
                }
            }

            prevButton.addEventListener('click', function(e) {
                e.preventDefault();
                prevStep();
            });
            
            nextButton.addEventListener('click', function(e) {
                e.preventDefault();
                nextStep();
            });

            steps.forEach(function(step) {
                step.querySelector('.qor-wizard__step-header').addEventListener('click', function() {
                    let clickedStep = parseInt(step.dataset.step);
                    if (clickedStep <= currentStep) {
                        currentStep = clickedStep;
                        showStep(currentStep);
                    }
                });

                step.querySelector('.qor-wizard__edit-icon').addEventListener('click', function(e) {
                    e.stopPropagation();
                    let clickedStep = parseInt(step.dataset.step);
                    if (clickedStep < currentStep) {
                        currentStep = clickedStep;
                        showStep(currentStep);
                    }
                });
            });

            $element.addEventListener('keypress', function(e) {
                if (e.key === 'Enter') {
                    e.preventDefault();
                }
            });

            showStep(currentStep);
        },

        destroy: function() {
            this.unbind();
            this.$element.removeData(NAMESPACE);
        },

        unbind: function() {
            this.$element.off(EVENT_DISABLE);
        }
    };

    QorWizard.DEFAULTS = {};

    QorWizard.plugin = function(options) {
        return this.each(function() {
            let $this = $(this);
            let data = $this.data(NAMESPACE);
            let fn;

            if (!data) {
                $this.data(NAMESPACE, (data = new QorWizard(this, options)));
            }

            if (typeof options === 'string' && $.isFunction(fn = data[options])) {
                fn.call(data);
            }
        });
    };

    $(function() {
        let selector = '[data-toggle="qor.wizard"]';
        $(document)
            .on(EVENT_DISABLE, function(e) {
                QorWizard.plugin.call($(selector, e.target), 'destroy');
            })
            .on(EVENT_ENABLE, function(e) {
                QorWizard.plugin.call($(selector, e.target));
            })
            .triggerHandler(EVENT_ENABLE);
    });

    return QorWizard;
});

