/**
 * @jest-environment jsdom
 */

// Mock getClientRects, offsetWidth, and offsetHeight
Object.defineProperty(HTMLElement.prototype, 'offsetWidth', {
    get: function() { return 100; },
});

Object.defineProperty(HTMLElement.prototype, 'offsetHeight', {
    get: function() { return 20; },
});

Object.defineProperty(HTMLElement.prototype, 'getClientRects', {
    value: function() { return [ { width: 100, height: 20 } ]; },
});

Object.defineProperty(HTMLElement.prototype, 'innerText', {
    get() {
        return this.textContent;
    },
    set(value) {
        this.textContent = value;
    },
});

const $ = require('jquery');

// Mock necessary components
$.fn.modal = jest.fn(); // Mock any jQuery plugins if necessary
HTMLFormElement.prototype.submit = jest.fn();

// Mock the global window object
global.window = {
    URL: {
        createObjectURL: jest.fn(() => 'blob:mocked-url'),
        revokeObjectURL: jest.fn(),
    },
};

const QorWizard = require('./qor-vertical-wizard.js');

describe('QorWizard Plugin', () => {
    let element;
    let qorWizardInstance;

    beforeEach(() => {
        document.body.innerHTML = `
        <form>
            <div class="qor-wizard__container" data-toggle="qor.wizard">
                <div class="qor-wizard__content">
                    <div class="qor-wizard__step is-visible is-active" data-step="0">
                        <div class="qor-wizard__step-header">
                            <div class="qor-wizard__step-header-left">
                                <div class="qor-wizard__step-number">1</div>
                                <div class="qor-wizard__step-label">Step 1</div>
                            </div>
                            <div class="qor-wizard__edit-icon"></div>
                        </div>
                        <div class="qor-wizard__step-summary"></div>
                        <div class="qor-wizard__step-content">
                            <div class="tyk-form-group tyk-form-group--default tyk-form-group--rounded-corners show-in-summary summary-inline" data-summary-label-class="label" data-summary-value-class=".radio-group">
                                <label>Radio Options</label>
                                <div class="radio-group" data-trigger-visibility="true" data-required="true">
                                    <div class="radio-option">
                                        <input type="radio" name="step1-option" id="step1_option1" value="Option1">
                                        <label class="radio-label" for="step1_option1">
                                            <span>Option 1</span>
                                        </label>
                                    </div>
                                    <div class="radio-option">
                                        <input type="radio" name="step1-option" id="step1_option2" value="Option2">
                                        <label class="radio-label" for="step1_option2">
                                            <span>Option 2</span>
                                        </label>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                    <div class="qor-wizard__step" data-step="1">
                        <div class="qor-wizard__step-header">
                            <div class="qor-wizard__step-header-left">
                                <div class="qor-wizard__step-number">2</div>
                                <div class="qor-wizard__step-label">Step 2</div>
                            </div>
                            <div class="qor-wizard__edit-icon"></div>
                        </div>
                        <div class="qor-wizard__step-summary"></div>
                        <div class="qor-wizard__step-content">
                            <div class="show-in-summary" data-summary-label-class="label" data-summary-value-class=".required-input-option1">
                                <label>Test Label</label>
                                <input type="text" data-required="true" class="required-input-option1" />
                            </div>
                            <div class="show-in-summary" data-summary-label-class="label" data-summary-value-class="input">
                                <label>Multiline Input</label>
                                <input type="text" value="Test Input 2" />
                            </div>
                        </div>
                    </div>
                    <div class="qor-wizard__step" data-step="2">
                        <div class="qor-wizard__step-header">
                            <div class="qor-wizard__step-header-left">
                                <div class="qor-wizard__step-number">3</div>
                                <div class="qor-wizard__step-label">Step 3</div>
                            </div>
                            <div class="qor-wizard__edit-icon"></div>
                        </div>
                        <div class="qor-wizard__step-summary"></div>
                        <div class="qor-wizard__step-content">
                            <div class="show-in-summary summary-inline" data-summary-label-class="label" data-summary-value-class=".radio-group">
                                <label>Step 3 Option</label>
                                <div class="radio-group" data-trigger-visibility="true" data-required="true">
                                    <div class="radio-option">
                                        <input type="radio" name="step3-option" id="step3_option1" value="Yes">
                                        <label class="radio-label" for="step3_option1">
                                            <span>Yes</span>
                                        </label>
                                    </div>
                                    <div class="radio-option">
                                        <input type="radio" name="step3-option" id="step3_option2" value="No">
                                        <label class="radio-label" for="step3_option2">
                                            <span>No</span>
                                        </label>
                                    </div>
                                </div>
                            </div>
                            </div>
                    </div>
                    <div class="qor-wizard__navigation">
                        <button type="button" class="tyk-button tyk-button--primary-outline tyk-button--md qor-wizard__button qor-wizard__prev" data-wizard-action="prev">Back</button>
                        <button type="button" class="tyk-button tyk-button--success tyk-button--md qor-wizard__button qor-wizard__next" data-wizard-action="next">Continue</button>
                    </div>
                </div>
            </div>
        </form>
    `;

        element = $('[data-toggle="qor.wizard"]');

        // Initialize QorWizard
        element.qorWizard = QorWizard.plugin;
        element.qorWizard();
        qorWizardInstance = element.data('qor.wizard');
    });

    afterEach(() => {
        if (qorWizardInstance) {
            qorWizardInstance.destroy();
        }
        document.body.innerHTML = '';
        jest.restoreAllMocks();
    });

    test('should initialize QorWizard correctly', () => {
        expect(qorWizardInstance).toBeDefined();
        expect(qorWizardInstance.$element[0]).toBe(element[0]);
        expect(qorWizardInstance.options).toEqual({});
    });

    test('should prevent navigation to next step with invalid input', () => {
        element.find('.qor-wizard__next').click();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(0);
    });

    test('should prevent form submission with invalid inputs on last step', () => {
        const submitMock = jest.fn((e) => e.preventDefault());
        element.closest('form').on('submit', submitMock);
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.qor-wizard__next').click();
        element.find('.qor-wizard__next').click();
        expect(submitMock).not.toHaveBeenCalled();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(1);
    });

    test('should handle prevStep correctly', () => {
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.qor-wizard__prev').click();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(0);
        expect(element.find('.qor-wizard__prev').hasClass('is-visible')).toBe(true);
    });

    test('should properly destroy the plugin', () => {
        const $prevButton = element.find('.qor-wizard__prev');
        const $nextButton = element.find('.qor-wizard__next');
        qorWizardInstance.destroy();
        $prevButton.click();
        $nextButton.click();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(0);
        expect(element.data('qor.wizard')).toBeUndefined();
    });

    test('should submit form when all inputs are valid on last step', () => {
        const submitMock = jest.fn((e) => e.preventDefault());
        element.closest('form').on('submit', submitMock);
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.required-input-option1').val('Valid Input').trigger('input');
        element.find('.qor-wizard__next').click();
        element.find('input[name="step3-option"][value="Yes"]').prop('checked', true).trigger('change');
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(2);
    });

    test('should update step summary correctly', () => {
        console.log('Starting the test');
    
        // Navigate to the second step
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        console.log('Navigated to second step');
    
        // Fill in the required input
        element.find('.required-input-option1').val('Test Input').trigger('input');
        console.log('Filled in required input:', element.find('.required-input-option1').val());
    
        // Navigate to the next step to trigger summary update
        element.find('.qor-wizard__next').click();
        console.log('Navigated to third step');
    
        // Navigate back to see the summary
        element.find('.qor-wizard__prev').click();
        console.log('Navigated back to second step');
    
        // Check if the summary for the second step is updated
        const summaryContainer = element.find('.qor-wizard__step[data-step="1"] .qor-wizard__step-summary');
        console.log('Summary container text:', summaryContainer.text());
    
        expect(summaryContainer.text()).toContain('Test Label | Test Input');
    });

    test('should handle step header clicks correctly', () => {
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.required-input-option1').val('Test Input').trigger('input');
        element.find('.qor-wizard__next').click();
        element.find('.qor-wizard__step[data-step="0"] .qor-wizard__step-header').click();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(0);
    });

    test('should handle edit icon clicks correctly', () => {
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.required-input-option1').val('Test Input').trigger('input');
        element.find('.qor-wizard__next').click();
        element.find('.qor-wizard__step[data-step="1"] .qor-wizard__edit-icon').click();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(1);
    });

    test('should handle radio group validation correctly', () => {
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.required-input-option1').val('Test Input').trigger('input');
        element.find('.qor-wizard__next').click();
        element.find('.qor-wizard__next').click();
        expect(element.find('.qor-wizard__step.is-active').data('step')).toBe(2);
    });

    test('should handle form submission correctly', () => {
        const submitMock = jest.fn((e) => e.preventDefault());
        element.closest('form').on('submit.qor.slideout', submitMock);
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        element.find('.required-input-option1').val('Test Input').trigger('input');
        element.find('.qor-wizard__next').click();
        element.find('input[name="step3-option"][value="Yes"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        expect(submitMock).toHaveBeenCalled();
    });

    test('should update step summary correctly for radio buttons', () => {
        // Select a radio option
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
    
        const summaryContainer = element.find('.qor-wizard__step[data-step="0"] .qor-wizard__step-summary');
        const inlineSummary = summaryContainer.find('.qor-wizard__inline-summary');
    
        expect(inlineSummary.length).toBe(1);
        expect(inlineSummary.css('display')).toBe('flex');
        expect(inlineSummary.css('flex-wrap')).toBe('wrap');
        expect(inlineSummary.css('gap')).toBe('60px');
    
        const summaryItems = inlineSummary.find('.qor-wizard__summary-item');
        expect(summaryItems.length).toBe(1);
        expect(summaryItems.eq(0).text()).toBe('Radio Options | Option 1');
    });


    test('should handle multiple inline summary items', () => {
        // Add another inline summary item to the first step
        const additionalInput = $('<div class="show-in-summary summary-inline" data-summary-label-class=".summary-label" data-summary-value-class=".additional-input"><label class="summary-label">Additional Label</label><input type="text" class="additional-input" /></div>');
        element.find('.qor-wizard__step[data-step="0"] .qor-wizard__step-content').append(additionalInput);

        // Fill in both inputs
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.additional-input').val('Additional Input').trigger('input');
        element.find('.qor-wizard__next').click();

        // Navigate back to see the summary
        element.find('.qor-wizard__prev').click();

        const summaryContainer = element.find('.qor-wizard__step[data-step="0"] .qor-wizard__step-summary');
        const inlineSummary = summaryContainer.find('.qor-wizard__inline-summary');

        expect(inlineSummary.length).toBe(1);
        const summaryItems = inlineSummary.find('.qor-wizard__summary-item');
        expect(summaryItems.length).toBe(2);
        expect($(summaryItems[0]).text()).toBe('Radio Options | Option 1');
        expect($(summaryItems[1]).text()).toBe('Additional Label | Additional Input');
    });

    test('should update step summary correctly for multiline summaries', () => {
        // Navigate to the second step
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
    
        // Fill in the required input and textarea
        element.find('.required-input-option1').val('Test Input').trigger('input');
        // Navigate to the next step to trigger summary update
        element.find('.qor-wizard__next').click();
    
        const summaryContainer = element.find('.qor-wizard__step[data-step="1"] .qor-wizard__step-summary');
        // Check multiline summary
        const multilineSummary = summaryContainer.contents().filter(function() {
            return this.nodeType === Node.TEXT_NODE || this.nodeName === 'BR';
        });
    
        expect(multilineSummary.length).toBe(4); // 3 text nodes + 4 <br> elements
        expect($(multilineSummary[0]).text()).toBe("Test Label | Test Input");
        expect($(multilineSummary[2]).text()).toBe("Multiline Input | Test Input 2");
    });

    test('should add error-highlight class to invalid required inputs', () => {
        // Navigate to second step without filling required input
        element.find('input[name="step1-option"][value="Option1"]').prop('checked', true).trigger('change');
        element.find('.qor-wizard__next').click();
        
        // Try to proceed without filling required input
        element.find('.qor-wizard__next').click();
        
        // Check if error-highlight class was added
        const requiredInput = element.find('.required-input-option1');
        expect(requiredInput.hasClass('error-highlight')).toBe(true);
    });

    test('should validate select2 inputs correctly', () => {
        // Add a select2 input to the first step
        const select2Input = $(`
            <div data-required="true">
                <span class="select2-selection__rendered"></span>
            </div>
        `);
        element.find('.qor-wizard__step[data-step="0"] .qor-wizard__step-content').append(select2Input);

        // Try to proceed without selecting an option
        element.find('.qor-wizard__next').click();

        // Check if error-highlight class was added to parent
        expect(select2Input.hasClass('error-highlight')).toBe(true);
    });
});